#!/usr/bin/perl -w
# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

use FindBin;
use lib "$FindBin::Bin/plugins", "$FindBin::Bin/lib";
use strict;
use ESP::SMS::Plugin 'load';
use Cozy::DateTime;
$| = 1;
umask 077;

my $C = ESP::SMS::Plugin->config;
my $DIR_BASE = $C->{DIR_BASE};
my $DIR_LOCAL = $C->{DIR_LOCAL};
my $BACKUPS_EXPIRES = $C->{BACKUPS_EXPIRES}||864000; # 10days
my $DIR_BACKUPS = $C->{DIR_BACKUPS}||"$DIR_LOCAL/backups";
my $DB_TYPE = $C->{DB_TYPE};
my $DB_NAME = $C->{DB_NAME};
my $DB_HOST = $C->{DB_HOST};
my $DB_PORT = $C->{DB_PORT};
my $DB_USER = $C->{DB_USER};
my $DB_PASS = $C->{DB_PASS};
my $datetime = Cozy::DateTime->new->datetime("%04d-%02d-%02dT%02d-%02d-%02d");
my $TAR = find_path('tar') or die "Not found command 'tar'";
my $TAR_FILE = "$DIR_BACKUPS/$datetime.tar.gz";
my $DUMP_FILE = "$DIR_BACKUPS/$datetime";
my $DUMP_FILE2 = $DUMP_FILE;
my $GZIP = find_path('gzip');
my $PG_REINDEX = 0;

my ($CMD_DUMP, $CMD_DUMP2, $CMD_CHECK, $CMD_INDEX);
my $CONNECTION = "";
my $PRINT_CONNECTION = "";
# PostgreSQL
if ($DB_TYPE eq 'Pg') {
	$CMD_DUMP = find_path('pg_dump') or die "Not found command 'pg_dump'";
	$CMD_DUMP2 = $CMD_DUMP;
	$CMD_CHECK = find_path('vacuumdb') or die "Not found command 'vacuumdb'";
	$CMD_INDEX = find_path('reindexdb') or die "Not found command 'reindexdb'";
	$CMD_DUMP .= ' -Fc'; # バイナリ形式
	$CMD_DUMP2 .= ' -Fp'; # テキスト形式
	$CMD_CHECK .= ' -q -z';
	$CMD_INDEX .= ' -q';
	$DUMP_FILE .= '.pgdump';
	$DUMP_FILE2 .= '.pgdump2';
	if ($DB_HOST) {
		$CONNECTION .= " --host=$DB_HOST";
		$PRINT_CONNECTION .= " --host $DB_HOST";
	}
	if ($DB_PORT) {
		$CONNECTION .= " --port=$DB_PORT";
		$PRINT_CONNECTION .= " --port $DB_PORT";
	}
	if ($DB_USER) {
		$CONNECTION .= " -U $DB_USER";
		$PRINT_CONNECTION .= " -U $DB_USER";
	}
	$ENV{PGPASSWORD} = $DB_PASS if $DB_PASS;
	$ENV{PGCLIENTENCODING} = 'UNICODE';
}
# MySQL
elsif ($DB_TYPE eq 'mysql') {
	$CMD_DUMP = find_path('mysqldump') or die "Not found command 'mysqldump'";
	$CMD_CHECK = find_path('mysqlcheck') or die "Not found command 'mysqlcheck'";
	$CMD_DUMP .= ' --opt --default-character-set=utf8';
	$CMD_CHECK .= ' -o';
	$DUMP_FILE .= '.mysqldump';
	if ($DB_HOST) {
		$CONNECTION .= " --host=$DB_HOST";
		$PRINT_CONNECTION .= " --host=$DB_HOST";
	}
	if ($DB_PORT) {
		$CONNECTION .= " --port=$DB_PORT";
		$PRINT_CONNECTION .= " --port=$DB_PORT";
	}
	if ($DB_USER) {
		$CONNECTION .= " --user=$DB_USER";
		$PRINT_CONNECTION .= " --user=$DB_USER";
	}
	if ($DB_PASS) {
		$CONNECTION .= " --password=$DB_PASS";
		$PRINT_CONNECTION .= " --password=****";
	}
}
else {
	die "$C->{DIR_PLUGINS}/config.pl: no defined DB_TYPE=('Pg' or 'mysql')";
}

print "$CMD_DUMP $PRINT_CONNECTION $DB_NAME > $DUMP_FILE\n";
!system "$CMD_DUMP $CONNECTION $DB_NAME > $DUMP_FILE" or die $?;
if ($GZIP && -e $DUMP_FILE) {
	print "$GZIP $DUMP_FILE\n";
	!system "$GZIP $DUMP_FILE" or die $?;
}

if ($CMD_DUMP2 && $DUMP_FILE2) {
	print "$CMD_DUMP2 $PRINT_CONNECTION $DB_NAME > $DUMP_FILE2\n";
	!system "$CMD_DUMP2 $CONNECTION $DB_NAME > $DUMP_FILE2" or die $?;
	if ($GZIP && -e $DUMP_FILE2) {
		print "$GZIP $DUMP_FILE2\n";
		!system "$GZIP $DUMP_FILE2" or die $?;
	}
}

print "$CMD_CHECK $PRINT_CONNECTION $DB_NAME\n";
!system "$CMD_CHECK $CONNECTION $DB_NAME" or die $?;

if ($PG_REINDEX && $CMD_INDEX) {
	print "$CMD_INDEX $PRINT_CONNECTION $DB_NAME\n";
	!system "$CMD_INDEX $CONNECTION $DB_NAME" or die $?;
}

print "$TAR --exclude $DIR_BACKUPS -zcf $TAR_FILE $DIR_BASE\n";
!system "$TAR --exclude $DIR_BACKUPS -zcf $TAR_FILE $DIR_BASE" or die $?;

clean_backups($DIR_BACKUPS, $BACKUPS_EXPIRES);

BEGIN {
	sub find_path {
		my $name = shift;
		my $cmd;
		for my $path (split(/:/, $ENV{PATH}), qw(/sbin /usr/sbin /usr/local/sbin)) {
			-x "$path/$name" and $cmd = "$path/$name", last;
		}
		$cmd;
	}
	sub clean_backups {
		my ($dir, $expires) = @_;
		if ($expires) {
			$expires = Cozy::DateTime->new(time-$expires)->date("%04d%02d%02d");
			my $expdays = $BACKUPS_EXPIRES / 86400;
			print "Expires: $expdays day(s): < $expires\n";
		} else {
			print "Expires: unlimited\n";
		}
		my $size = 0;
		my $total = 0;
		my $count = 0;
		opendir my $dh, $dir or die "Can not open : $dir : $!";
		while (my $name = readdir $dh) {
			chomp $name;
			if (my ($y, $m, $d) = $name =~ /^(\d+)\-(\d+)\-(\d+)T\d+\-\d+\-\d+\./) {
				my $file = "$dir/$name";
				$total++;
				if ($expires && "$y$m$d" < $expires) {
					if (unlink $file) {
						$count++;
						print "Removed: $file\n";
					} else {
						print "Can not removed: $file : $!\n";
						$size += -s $file;
					}
				} else {
					$size += -s $file;
				}
			}
		}
		closedir $dh;
		print "Total: $count/$total file(s) removed.\n";
		printf("Backup Size: %.2f M byte(s)\n", $size/1024/1024);
	}
}

__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
