#!/usr/bin/perl -w
# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $
#
# サーバ情報表示CGI
#
# この CGI はサーバ情報を表示します。
# サーバ情報が不明の場合に活用して下さい。
#
# ●設置方法
# 1. このファイルを www ディレクトリに置きます。
# 2. このファイルの冒頭(1行目)にある Perl のパスを設定します。
# 3. このファイルの属性を実行可能(パーミッションを 755)に設定します。
# 4. ブラウザで check.cgi/all にアクセスします。
#    アクセスする時は、check.cgi の URL の最後に /all を付けて下さい。
# 5. 使用後は、このファイルをブラウザでアクセスできないディレクトリに
#    移動するか、ファイルの属性を実行不可(パーミッション 644)に設定し
#    て下さい。
#
# ●注意
# この CGI を実行するとサーバの情報を容易に知ることができます。
# 設置したままにしておくと、他人に悪用される危険性がありますので、
# 使わない場合は、必ず削除するか実行不可にしアクセスできないようにして下さい。
# CGI 設置時にのみ使用されることをお勧め致します。

use strict;
$|=1;

my @MOD = (
		   'Encode'      => 2.10,
		   'Digest::MD5' => 2.20,
		   'DBI'         => 1.48,
		   'DBD::Pg'     => 1.43,
		   'DBD::mysql'  => 4.0
		  );

my $info = $ENV{PATH_INFO}||'';
$info =~ tr/a-z//dc;
if ($info ne 'all') {
	output([{ name => '=/エラー/=', value => '=/URLの最後に /(キーワード) を追加してアクセスして下さい。/=' }]);
	exit;
}
my @env;
my $pwd = `pwd`;
$pwd ||= `/bin/pwd`;
$pwd ||= `/usr/bin/pwd`;
chomp $pwd;
my $uname = `uname -a`;
$uname ||= `/bin/uname -a`;
$uname ||= `/usr/bin/uname -a`;
$uname ||= $^O;
chomp $uname;
my $ver = $^V ? sprintf("%vd", $^V) : $];
eval("use 5.8.0;");
if ($@) {
	$ver .= "\n";
	$ver .= '=/プログラムが動作しない可能性があります。/=';
	$ver .= "\n=/エラー: $@/=";
}
my $inc = join "\n", @INC;

push @env, { name => 'このCGIがあるディレクトリの絶対パス', value => $pwd };
push @env, { name => 'このCGIのURL', value => 'http://'.$ENV{SERVER_NAME}.$ENV{REQUEST_URI} };
push @env, { name => 'OS', value => $uname };
push @env, { name => 'WWWサーバ', value => $ENV{SERVER_SOFTWARE}||'' };
push @env, { name => 'Perlのバージョン(5.8以上)', value => $ver };
push @env, { name => 'PerlのINCパス', value => $inc };

my $N = scalar @MOD;
my $i = 0;
while ($i < $N) {
	my $mod = $MOD[$i++];
	my $ver = $MOD[$i++];
	my $module = $ver ? "$mod $ver 以上" : $mod;
	eval("use $mod" . ($ver ? " $ver;" : ";"));
	if ($@) {
		my $error = "利用できません。\n";
		$error .= '=/';
		$error .= 'データベースにPostgreSQLをご利用の場合は' if $mod =~ /::Pg/;
		$error .= 'データベースにMySQLをご利用の場合は' if $mod =~ /::mysql/;
		$error .= 'プログラムが動作しない可能性があります。';
		$error .= '/=';
		push @env, { name => $module, value => $error."\n=/エラー: $@/=" };
	} else {
		push @env, { name => $module, value => 'バージョン'.$mod->VERSION.'が利用可能です。' };
	}
}
push @env, { name => 'mod_perlのバージョン', value => $ENV{MOD_PERL}||'' };
$ENV{REMOTE_HOST} = gethostbyaddr(pack('C4',split(/\./,$ENV{REMOTE_ADDR})),2) if $ENV{REMOTE_ADDR};
push @env, { name => '接続元ホスト名', value => $ENV{REMOTE_HOST}||'' };
push @env, { name => '接続元IPアドレス', value => $ENV{REMOTE_ADDR}||'' };
push @env, { name => '接続元ブラウザ情報', value => $ENV{HTTP_USER_AGENT}||'' };

output(\@env);
exit;

sub output {
	my $envref = shift;
	print <<'EOF';
Content-Type: text/html

<html>
<head>
<meta HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">
<style type="text/css">
<!--
body {
	font-size: 12px;
	color: #000000;
	line-height: 24px;
	text-align: left;
	background-color: #FFFFFF;
}
table {
	border-collapse: collapse;
}
table th,td {
	padding: 4px;
	border: 1px solid #666666;
	border-color: #666666;
	font-size: 12px;
	font-weight: normal;
}
strong {
	white-space: nowrap;
	font-weight: bold;
}
span {
	color: #C33;
}
.odd {
	background-color: #EEEEEE;
}
-->
</style>
</head>
<body bgcolor="#ffffff">
<h2>サーバ情報の確認(E-Shoppy Mall V4)<h2>
<table>
EOF
	my $i = 0;
	for my $env (@$envref) {
		my $name = escape($env->{name});
		my $value = escape($env->{value});
		if ($i++ % 2) {
			print qq|<tr><td><strong>$name</strong></td><td>$value</td></tr>\n|;
		} else {
			print qq|<tr><td class="odd"><strong>$name</strong></td><td class="odd">$value</td></tr>\n|;
		}
	}
	print <<'EOF';
</table>
</body>
</html>
EOF
}

sub escape {
	my $str = shift;
	return "" unless defined $str;
	$str =~ s|&|&amp;|g  if $str =~ /&/;
	$str =~ s|"|&quot;|g if $str =~ /"/;
	$str =~ s|'|&#39;|g  if $str =~ /'/;
	$str =~ s|<|&lt;|g   if $str =~ /</;
	$str =~ s|>|&gt;|g   if $str =~ />/;
	$str =~ s|\r?\n|<br />|g if $str =~ /\r?\n/;
	$str =~ s|=/|<span>|g if $str =~ /=\//;
	$str =~ s|/=|</span>|g if $str =~ /\/=/;
	$str;
}

# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
