# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

package ESP::SMS::Plugin::DataFilter;

use strict;
our $VERSION = '0.02';

use Cozy::base qw(Cozy::DataFilter);
use Cozy::DataFilter::Plugin::Expr;
use Cozy::DataFilter::Plugin::HTML;
use Cozy::DataFilter::Plugin::Email;
use Cozy::DataFilter::Plugin::Misc;
use Cozy::DataFilter::Plugin::Text;
use Cozy::DataFilter::Plugin::Data::Config;
use Cozy::DataFilter::Plugin::Data::DateTime;
use Cozy::DataFilter::Plugin::Data::Extract;
use Cozy::DataFilter::Plugin::Data::By;

# 画像URL
# <% var.imgurl %>
sub filter_imgurl {
	$_[0]->plugin->config->{INC_IMAGE}.'/'.$_[1] if defined $_[1];
}

# 在庫数の文字表示
# <% gds_stock.order %>
# 0個       :「売り切れ」
# 1個〜10個 :「残りあと僅か」
# 11個〜    :「注文可」
sub filter_order {
	use utf8;
	return '売り切れ' if $_[1] <=  0;
	return '残りあと僅か' if $_[1] <= 10;
	return '注文可';
}

# 年代
# <% rev_age.age_name %>
sub filter_age_name {
	use utf8;
	return unless defined $_[1];
	return '10代' if $_[1] == 1;
	return '70代以上' if $_[1] >= 7;
	return $_[1].'0代';
}

# 消費税等
# <% var.tax(gds_intax) %>
# <% var.tax %>
sub filter_tax {
	my ($self, $price, $field) = @_;
	my $rate = $self->plugin->config->{GDS_TAXRATE};
	my $gds_tprice = defined $field && $self->{scope}->{stash}->{$field} ?
		$price * ($rate / (1 + $rate)) : $price * $rate;
	int("$gds_tprice");
}

# 税込み価格
# <% var.with_tax(gds_intax) %>
# <% var.with_tax %>
sub filter_with_tax {
	my ($self, $price, $field) = @_;
	return $price if defined $field && $self->{scope}->{stash}->{$field};
	my $gds_tprice = $price * $self->plugin->config->{GDS_TAXRATE}; # 消費税等
	$price + int("$gds_tprice");
}

# 税抜き価格
# <% var.without_tax(gds_intax) %>
# <% var.without_tax %>
sub filter_without_tax {
	my ($self, $price, $field) = @_;
	return $price if defined $field && !$self->{scope}->{stash}->{$field};
	my $rate = $self->plugin->config->{GDS_TAXRATE};
	my $gds_tprice = $price * ($rate / (1 + $rate)); # 消費税等
	$price - int("$gds_tprice");
}

# 性別名
# <% var.sex_name %>
sub filter_sex_name {
	use utf8;
	return '' unless defined $_[1];
	return '女性' if $_[1] == 1;
	return '男性' if $_[1] == 2;
	return '';
}

# K/M/G/T単位付与
# <% var.unit %>
sub filter_unit {
	my ($self, $str) = @_;
	return $str unless defined $str && $str =~ /^[0-9]+$/ && $str >= 1024;
	my $u;
	for my $s (qw(K M G T)) {
		$u = $s;
		$str /= 1024;
		last if $str < 1024;
	}
	return sprintf("%.1f$u", $str);
}

# Wiki
# <% var.wiki %>
sub filter_wiki {
	my $self = shift;
	$self->plugin->new('Wiki')->filter(@_);
}

# SafeHTMLフィルター
# <% var.safe_html %>
# <% var.safe_html(body,a,font) %>
sub filter_safe_html {
	my $self = shift;
	$self->plugin->new('SafeHTML')->filter(@_);
}

# <% shp_email.atmark %>
# <% shp_email.atmark(＠) %>
sub filter_atmark {
	my ($self, $str, $mark) = @_;
	return unless defined $str;
	return $str unless $str =~ /\@/;
	$mark ||= ' _at_ ';
	$str =~ s/\@/$mark/;
	return $str;
}

# <% cgi_order.https %>
sub filter_https {
	my ($self, $str) = @_;
	return unless defined $str;
	return $str unless $str =~ /^http:/;
	my $ssl_regexp = $self->plugin->config->{SSL_REGEXP};
	return $ssl_regexp ? &$ssl_regexp($str) : $str;
}

# 半角カナを全角に変換
# <% ctm_kana1.hk2z %>
sub filter_hk2z {
	my ($self, $str) = @_;
	return unless defined $str;
	use utf8;
	$str =~ tr/ｱｲｳｴｵｶｷｸｹｺｻｼｽｾｿﾀﾁﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏﾐﾑﾒﾓﾔﾕﾖﾗﾘﾙﾚﾛﾜｦﾝ/アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヲン/;
	return $str;
}

# キロ単位に変換
# <% gds_weight.kilo %>
# sub filter_kilo {
#	my ($self, $str) = @_;
#	return $str unless defined $str && $str =~ /^[0-9]+$/ && $str >= 1000;
#	return $str / 1000;
# }

#
# フィルターの追加例
#
# 値にコンマを付けるフィルター
# sub filter_comma {
#	my ($self, $str) = @_;
#	$str =~ s/([0-9]{1,3})(?=(?:[0-9][0-9][0-9])+(?![0-9]))/$1,/g
#		if defined $str && $str =~ /^[0-9]+$/ && $str >= 1000;
#	$str;
#}
#
# テンプレートの記述例(1)
# 変数 price の値にコンマを付けて表示する。
#
# 価格: <% price.comma %>
#
# テンプレートの記述例(2)
# 直接書いた数字にコンマを付けて表示する。
#
# 価格: <% BLOCK .comma %>12345<% END %>
#
# 表示例
#
# 価格: 12,345
#

# 携帯コンテンツフィルター
# キャリアごとに入力モードの設定を書換える。
#
# 記述例(DoCoMo用のistyleで記述すること)
# <form action="">
# <% BLOCK .mobile -%>
# <input type="text" name="foo" value="" istyle="3" />
# <input type="submit" value="送信" />
# <% END -%>
# </form>
#
# DoCoMo用の出力例(DoCoMo/2.0)
# <form action="">
# <input type="text" name="foo" value="" istyle="3" style="-wap-input-format:&quot;*&lt;ja:en&gt;&quot;;" />
# <input type="submit" value="送信" />
# </form>
#
# au用の出力例(KDDI/UP.Browser)
# <form action="">
# <input type="text" name="foo" value="" format="*m" style="-wap-input-format:*m;" />
# <input type="submit" value="送信" />
# </form>
#
# SoftBank用の出力例(Softbank/J-PHONE/Vodafone)
# <form action="">
# <input type="text" name="foo" value="" mode="alphabet" />
# <input type="submit" value="送信" />
# </form>
#
my %FILTER_MOBILE =
	(
	 # docomo
	 # 1:全角かな/2:半角カナ/3:英字/4:数字
	 DO => {
			1 => qq|istyle="1" style="-wap-input-format:&quot;*&lt;ja:h&gt;&quot;;"|,
			2 => qq|istyle="2" style="-wap-input-format:&quot;*&lt;ja:hk&gt;&quot;;"|,
			3 => qq|istyle="3" style="-wap-input-format:&quot;*&lt;ja:en&gt;&quot;;"|,
			4 => qq|istyle="4" style="-wap-input-format:&quot;*&lt;ja:n&gt;&quot;;"|,
		   },
	 # au
	 AU => {
			1 => qq|format="*M" style="-wap-input-format:*M;"|,
			2 => qq|format="*M" style="-wap-input-format:*M;"|,
			3 => qq|format="*m" style="-wap-input-format:*m;"|,
			4 => qq|format="*x" style="-wap-input-format:*x;"|, # TELのために記号入力可能
		   },
	 # softbank
	 SO => {
			1 => qq|istyle="1" mode="hiragana"|,
			2 => qq|istyle="2" mode="katakana"|,
			3 => qq|istyle="3" mode="alphabet"|,
			4 => qq|istyle="4" mode="numeric"|,
		   },
	);
sub filter_mobile {
	my ($self, $str, $args) = @_;
	return '' unless defined $str && !ref $str && defined $ENV{HTTP_USER_AGENT};
	if ($ENV{HTTP_USER_AGENT} =~ /^DoCoMo/) {
		$str =~ s/ istyle="([1-4])"/ $FILTER_MOBILE{DO}->{$1}/g;
	}
	elsif ($ENV{HTTP_USER_AGENT} =~ /^(?:KDDI|UP\.Browser)/) {
		$str =~ s/ istyle="([1-4])"/ $FILTER_MOBILE{AU}->{$1}/g;
	}
	elsif ($ENV{HTTP_USER_AGENT} =~ /^(?:Softbank|Vodafone|J-PHONE|MOT-)/) {
		$str =~ s/ istyle="([1-4])"/ $FILTER_MOBILE{SO}->{$1}/g;
	}
	return $str;
}

# 商品表示SSI実行
# <% .Search(args) %> or
# <% TMPL .Search(args) %>template tags<% END %>
sub filter_Search {
	my ($self, $str, $args) = @_;
	return '' if $self->{__pages_search};
	my $stash = $self->{scope}->{stash};
	$args =~ s/\$([a-zA-Z0-9_]+)/$$stash{$1}/g if $args =~ /\$[a-zA-Z0-9_]+/;
	my $output = '';
	my $app = $self->plugin->new('Search', { env => $self->env });
	$app->set(TMPLFILE => \$str) unless ref $str;
	$app->cgi->set_query_string($args);
	$app->tmpl->filter->{__pages_search} = 1; # 実行中
	$app->tmpl->handle(\$output);
	$app->run;
	return $output;
}

# 記事表示SSI実行
# <% .Articles(args) %> or
# <% TMPL .Articles(args) %>template tags<% END %>
sub filter_Articles {
	my ($self, $str, $args) = @_;
	return '' if $self->{__pages_articles};
	my $stash = $self->{scope}->{stash};
	$args =~ s/\$([a-zA-Z0-9_]+)/$$stash{$1}/g if $args =~ /\$[a-zA-Z0-9_]+/;
	my $output = '';
	my $app = $self->plugin->new('Articles', { env => $self->env });
	$app->set(TMPLFILE => \$str) unless ref $str;
	$app->cgi->set_query_string($args);
	$app->tmpl->filter->{__pages_articles} = 1; # 実行中
	$app->tmpl->handle(\$output);
	$app->run;
	return $output;
}

# スマートフォン用テンプレートの変数保存
# <% TMPL .cache(list) %>template tags<% END %>
sub filter_cache {
	my ($self, $str, $name) = @_;
	return '' unless defined $str;
	return '' unless ($self->{scope}->{param}->{cmd}||'') eq 'shtml';
	$name ||= 'list';
	if ($name =~ /^[a-zA-Z0-9_]+$/) {
		$self->{scope}->{param}->{$name} = $str;
	}
	$str;
}

# スマートフォン用改ページJavaScriptの生成
# <% list.jqmpager(#list) %>
sub filter_jqmpager {
	my ($self, $str, $id) = @_;
	return '' unless defined $str;
	return '' unless ($self->{scope}->{param}->{cmd}||'') eq 'shtml';
	return '' unless $id && $id =~ /^#[a-zA-Z0-9_]+$/;
	my $prefix = 'data';
	my $chunks = "   var html = [];\n   html.push(";
	for my $chunk (split(/(?=<%.+?%>)/, $str)) {
		if ($chunk =~ /^<%\s*
					   (IF|UNLESS)\s+				   # $1: IF|UNLESS
					   \$?([a-zA-Z0-9:][a-zA-Z0-9_]*)  # $2: 変数名(_で始まる変数名は不可)
					   .*?\s*\-?%>
					   (.*)						       # $3: IF内容
					   $/sx) {
			$chunks .= ");\n";
			$chunks .= $1 eq 'IF'
				? "if ($prefix.$2 != undefined && $prefix.$2 != '' && $prefix.$2 != '0' && $prefix.$2 != 0) {\n"
				: "if ($prefix.$2 == undefined || $prefix.$2 == '' || $prefix.$2 == '0' || $prefix.$2 == 0) {\n";
			$chunks .= '   html.push("'.jstring($3).'"';
		}
		elsif ($chunk =~ /^<%\s*ELSE\s*
						  \-?%>
						  (.*)			# $1: ELSE内容
						  $/sx) {
			$chunks .= ");\n} else {\n";
			$chunks .= '   html.push("'.jstring($1).'"';
		}
		elsif ($chunk =~ /^<%\s*(?:END|\/)[A-Z]*\s*
						  \-?%>
						  (.*)			# $1: その他
						  $/sx) {
			$chunks .= ");\n}\n";
			$chunks .= '   html.push("'.jstring($1).'"';
		}
		elsif ($chunk =~ /^<%\s*
						  FOR\s+
						  \@?[a-zA-Z0-9:][a-zA-Z0-9_]*
						  .*?\s*\-?%>
						  (.*)			# $1: FOR内容
						  $/sx) {
			$chunks .= ");\n{\n";
			$chunks .= '   html.push("'.jstring($1).'"';
		}
		elsif ($chunk =~ /^<%\s*
						  (?:VAR\s+)?
						  \$?
						  ([a-zA-Z0-9:][a-zA-Z0-9_]*)?  # $1: 変数名(_で始まる変数名は不可)
						  .*?\s*\-?%>
						  (.*)					        # $2: その他
						  $/sx) {
			$chunks .= " + $prefix.$1 + ";
			$chunks .= '"'.jstring($2).'"';
		} else {
			$chunks .= '"'.jstring($chunk).'"';
		}
	}
	$chunks .= ");\n   return html.join('');";
	return <<"EOF";
<script type="text/javascript">//<![CDATA[
\$(function(){
   \$('$id').Pager({ id: '#pager', render: function($prefix) {
$chunks
   }});
});
//]]>
</script>
EOF
}

sub jstring {
	my ($str) = @_;
	return '' unless defined $str;
	$str =~ s|\\|\\\\|g if $str =~ /\\/;
	$str =~ s|"|\\\"|g  if $str =~ /"/;
	$str =~ s|'|\\\'|g  if $str =~ /'/;
	$str =~ s|\r|\\r|g  if $str =~ /\r/;
	$str =~ s|\n|\\n|g  if $str =~ /\n/;
	$str;
}

1;
__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
