# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

package ESP::SMS::Plugin::Untaint;

use strict;
our $VERSION = '0.06';

use Cozy::base qw(Cozy::Untaint);
use Cozy::Untaint::Plugin::Config;
use Cozy::Untaint::Plugin::Text;
use Cozy::Untaint::Plugin::DateTime;

# ショップディレクトリ
sub is_shp_dir {
	return if $_[1] eq 'image' || $_[1] eq 'styles' || $_[1] eq 'js';
	$_[1] =~ /^[a-z][a-z0-9_-]+$/
}

# 日時
sub is_YYYYMMDDHH24MI {
	$_[1] =~ /^[0-9]{4}\/[0-9][0-9]?\/[0-9][0-9]?(?:\s+[0-9][0-9]?:[0-9][0-9])?$/
}

# ディレクトリパス
sub is_path {
	$_[1] =~ m<^[a-zA-Z0-9_][a-zA-Z0-9_/]*[a-zA-Z0-9_]$>
}

# ファイル名(ディレクトリ含む)
sub is_filename {
	$_[1] =~ /^[a-zA-Z0-9_\-]+(?:\.[a-zA-Z]+)?$/
}

# ファイル名(モール管理室の作成可能なファイル名)
sub is_mk_filename {
	return if $_[1] =~ /^[0-9_]+\./;
	$_[1] =~ /^[a-zA-Z0-9_\-]+(?:\.[a-z]+)?\.(?:html|shtml|xhtml|css|js|rdf)$/
}

# ファイル名(ショップ管理室の作成可能なファイル名)
sub is_mk_shop_filename {
	return if $_[1] =~ /^[0-9_]+\./;
	$_[1] =~ /^[a-zA-Z0-9_\-]+(?:\.[a-z]+)?\.(?:html|shtml|xhtml)$/
}

# ファイル内容(モール管理室の作成可能なファイル内容)
# skel/manage/webmgr/0/_error.yml用
sub is_mk_filetext {
	$_[1] =~ /^[\r\n\t\p{IsPrint}]+\z/
}

# ファイル内容(ショップ管理室の作成可能なファイル内容)
# skel/manage/webmgr/_error.yml用
sub is_mk_shop_filetext {
	my ($self, $str) = @_;
	$str =~ /^[\r\n\t\p{IsPrint}]+\z/ or return;
	my $html = $self->plugin->new('SafeHTML');
	$html->filter($str);
	use utf8;
	my $errstr = $html->errstr("%s行目%sタグ") or return 1;
	$self->error($errstr => 'invalid_tag');
	return;
}

# ファイル名(モール管理室のアップロード可能なファイル名)
# skel/manage/webmgr/0/_error.yml用
sub is_up_filename {
	return if $_[1] =~ /^[0-9_]+\./;
	$_[1] =~ /^[a-zA-Z0-9_\-]+(?:\.[a-z]+)?\.(?:html|shtml|xhtml|css|js|gif|GIF|jpg|JPG|png|PNG|ico|rdf)$/
}

# ファイル名(ショップ管理室のアップロード可能なファイル名)
# skel/manage/webmgr/_error.yml用
sub is_up_shop_filename {
	return if $_[1] =~ /^[0-9_]+\./;
	$_[1] =~ /^[a-zA-Z0-9_\-]+(?:\.[a-z]+)?\.(?:html|shtml|xhtml|gif|GIF|jpg|JPG|png|PNG)$/
}

# 画像ファイル(ショップ管理室のGIF/JPEG/PNGアップロード、商品画像登録)
sub is_shop_image_file {
	my ($self, $file, $i) = @_;
	return $self->error(tmp_filename => 'unsuitable') unless $file =~ /^\//;
	unless (-e $file && -s _ <= 512000) {
		use utf8;
		return $self->error("ファイル$i" => 'invalid_size');
	}
	open my $fh, '<', $file or do {
		warn "Can not open '$file': $!\n";
		return;
	};
	my $buffer;
	read($fh, $buffer, 128) or do {
		close $fh;
		return;
	};
	close $fh;
	return 1 if $buffer =~ /(?:GIF|......(?:JFIF|.xif))/;
	return 1 if $buffer =~ /^\x89PNG\r\n\x1a\n.*IHDR/;
	return;
}

# テキストファイル(ショップ管理室のHTML/SHTML/XHTMLアップロード)
sub is_shop_text_file {
	my ($self, $file, $i) = @_;
	return $self->error(tmp_filename => 'unsuitable') unless $file =~ /^\//;
	unless (-e $file && -s _ <= 512000) {
		use utf8;
		return $self->error("ファイル$i" => 'invalid_size');
	}
	open my $fh, '<', $file or do {
		warn "Can not open '$file': $!\n";
		return;
	};
	my $str = do { local $/; <$fh> };
	close $fh;
	$self->is_mk_shop_filetext($str);
}

# 複数メールアドレスとタグ(コンマ区切り)
# skel/manage/cmlmgr/_error.yml用
sub is_emails_or_tags {
	my ($self, $str) = @_;
	my $errors = 0;
	for my $chunk (split(/[,\r\n]+/, $str)) {
		$self->is_email($chunk)	or
			($chunk =~ /^(?:\s*<% (?:[a-z0-9_]+|IF [a-z0-9_]+|ELSE|END) %>)+$/ or $errors++);
	}
	return $errors ? 0 : 1;
}

# フリーメールアドレス不可のチェック
# skel/manage/shpmgr/_error.yml用
sub is_trusty_email {
	my ($self, $str) = @_;
	my ($s, $domain) = split '@', $str;
	return unless $domain;
	return if $domain =~ /^gmail\.com/i;
	return if $domain =~ /^yahoo\.co\.jp/i;
	return if $domain =~ /^hotmail\.co\.jp/i;
	return if $domain =~ /^nifmail\.jp/i;
	return if $domain =~ /^infoseek\.jp/i;
	return if $domain =~ /^excite\.co\.jp/i;
	return if $domain =~ /^mail\.goo\.ne\.jp/i;
	return if $domain =~ /^supermailer\.jp/i;
	return if $domain =~ /^vjp\.jp/i;
	return if $domain =~ /^compass\.jp/i;
	return if $domain =~ /^inter7\.jp/i;
	return if $domain =~ /^mb\.dreammail\.ne\.jp/i;
	return if $domain =~ /^1kw\.jp/i;
	return if $domain =~ /^netidol\.jp/i;
	return if $domain =~ /^csc\.jp/i;
	return if $domain =~ /^drive\.co\.jp/i;
	return if $domain =~ /^estyle\.ne\.jp/i;
	return if $domain =~ /^fact-mail\.com/i;
	return if $domain =~ /^hotmail\.com/i;
	return if $domain =~ /^kobej\.zzn\.com/i;
	return if $domain =~ /^pc_run\.zzn\.com/i;
	return if $domain =~ /^fubako\.com/i;
	return if $domain =~ /^smoug\.net/i;
	return if $domain =~ /^lycos\.com/i;
	return if $domain =~ /^otegami\.com/i;
	return if $domain =~ /^meritmail\.net/i;
	return if $domain =~ /^mini-blog\.net/i;
	return if $domain =~ /^melmel\.tv/i;
	return if $domain =~ /^keyakiclub\.net/i;
	return if $domain =~ /^saku2\.com/i;
	return if $domain =~ /^tera-hp\.net/i;
	return $self->is_email($str);
}

# リダイレクト先URLのチェック
sub is_next_url {
	my ($self, $str) = @_;
	if ($str =~ /^https?:\/\/([a-zA-Z0-9_\.!;$%&+\-\/=?~#]+)$/) {
		my $domain = $1;
		my $C = $self->plugin->config;
		my $allow = 0;
		# 許可するURLリスト
		for my $url ($C->{CGIURL_BASE}.'/customer/',
					 $C->{CGIURL_BASE}.'/order/',
					 $C->{CGIURL_BASE}.'/search/') {
			my ($d) = $url =~ /^https?:\/\/(.+)$/;
			$allow++ if defined $d && index($domain, $d) == 0;
		}
		return 1 if $allow;
	}
	return;
}

1;
__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
