#!/usr/bin/perl -w
# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

use FindBin;
use lib "$FindBin::Bin/plugins", "$FindBin::Bin/lib";
use strict;
use ESP::SMS::Plugin 'load';
use ESP::SMS::Data;
use utf8;
use Encode;
use Encode::JP::H2Z;
use warnings;
$| = 1;

my $C = ESP::SMS::Plugin->config;
my %OPTION = map { split /=/, } grep /--\w+/, @ARGV;
if ($OPTION{'--fetch_ken_all'}) {
	fetch_ken_all();
	exit;
}

# 登録
{
	my $dbh = ESP::SMS::Data->handler->dbh;
	my $file = $C->{DIR_BACKUPS}.'/ken_all.csv';
	-e $file or die "Not found postal file: $file\n";
	my $logfile = $FindBin::Bin.'/postal.log';
	my $i = 0;
	my $j = 0;
	my $max_postid = "";
	my $max_count = 0;
	print "Postcode: $file\n";
	my $total = 0;
	my %postcode;
	my %seen;

	# ●新郵便番号データファイルのフォーマット
	#  1.全国地方公共団体コード(JIS X0401、X0402)………　半角数字
	#  2.現郵便番号(5桁)………………………………………　半角数字
	#  3.新郵便番号(7桁)………………………………………　半角数字
	#  4.都道府県名　…………　半角カタカナ(コード順に掲載)　(注1)
	#  5.市区町村名　…………　半角カタカナ(コード順に掲載)　(注1)
	#  6.町域名　………………　半角カタカナ(五十音順に掲載)　(注1)
	#  7.都道府県名　…………　漢字(コード順に掲載)　(注1,2)
	#  8.市区町村名　…………　漢字(コード順に掲載)　(注1,2)
	#  9.町域名　………………　漢字(五十音順に掲載)　(注1,2)
	# 10.一町域が二以上の新郵便番号で表される場合の表示　(注3)　(「1」は該当、「0」は該当せず)
	# 11.小字毎に番地が起番されている町域の表示　(注4)　(「1」は該当、「0」は該当せず)
	# 12.丁目を有する町域の場合の表示　(「1」は該当、「0」は該当せず)
	# 13.一つの新郵便番号で二以上の町域を表す場合の表示　(注5)　(「1」は該当、「0」は該当せず)
	# 14.更新の表示（注6）（「0」は変更なし、「1」は変更あり、「2」廃止（廃止データのみ使用））
	# 15.変更理由　(「0」は変更なし、「1」市政・区政・町政・分区・政令指定都市施行、
	#			   「2」住居表示の実施、「3」区画整理、「4」郵便区調整、集配局新設、
	#				「5」訂正、「6」廃止(廃止データのみ使用))

	# ●変換後のデータファイルのフォーマット(タブ区切り)
	# 新郵便番号(7桁),都道府県番号(半角数字2桁),市区町村番号(半角数字3桁),
	# 都道府県名(漢字),市区町村名(漢字),町域名(漢字),都道府県名(全角カナ),市区町村名(全角カナ),町域名(全角カナ)

	# 地域情報取得(関東、中部、近畿などの名称)
	my %AREA;
	while (<DATA>) {
		last if /__END__/;
		next if /^$/;
		next if /^[\#\-\=]+/;
		chomp;
		my ($area_id, $state_id, $area_name, $state_name) = split(/,/);
		$AREA{$area_id} = $area_name;
	}
	# CSVデータ数
	{
		open my $fh, '<', $file or die "Can not open $file: $!";
		while (<$fh>) {
			last if /^\cZ/;
			$total++;
		}
		close $fh;
	}
	open my $fh, '<:encoding(cp932)', $file or die "Can not open $file: $!";
	# ログ保存
	no strict 'subs';
	my $logging = 0;
	$logging = 1 if $logfile && open my $stderr, '>&', STDERR;
	open STDERR, '>', $logfile if $logging;
	my $DATA_SOURCE = $dbh->get_info(2);
	print "Removing old Postcode from 'zip' table of $DATA_SOURCE:\n";
	$dbh->begin_work;
	$dbh->do('DELETE FROM zip');
	print "Copying new Postcode to 'zip' table of $DATA_SOURCE:\n";
	my $sth = $dbh->prepare('INSERT INTO zip (zip_id, zip_code, zip_are_id, zip_city_id, zip_are_name, zip_city_name, zip_town_name, zip_are_kana, zip_city_kana, zip_town_kana, zip_update, zip_create) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), NOW())');
	while (<$fh>) {
		last if /^\cZ/;
		my $tmp = Encode::encode('euc-jp', $_, Encode::FB_HTMLCREF);
		Encode::JP::H2Z::h2z(\$tmp);
		$tmp = Encode::decode('euc-jp', $tmp);
		$tmp =~ s/&#65374;/-/g; # 〜 => -
		$tmp =~ s/&#65293;/-/g; # ─ => -
		$tmp =~ tr/０-９/0-9/;
		warn Encode::encode('utf8', "escaped html: $tmp\n") if $tmp =~ /&#/;
		$tmp =~ s/(\r?\n)?$/,/;
		my @values = map {/^"(.*)"$/ ? scalar($_ = $1, s/""/"/g, $_) : $_}
			($tmp =~ /("[^"]*(?:""[^"]*)*"|[^,]*),/g);

		#  $values[0]: 全国地方公共団体コード(JIS X0401、X0402)………　半角数字
		#  $values[2]: 新郵便番号(7桁)………………………………………　半角数字
		#  $values[3]: 都道府県名　…………　半角カタカナ(コード順に掲載)　(注1)
		#  $values[4]: 市区町村名　…………　半角カタカナ(コード順に掲載)　(注1)
		#  $values[5]: 町域名　………………　半角カタカナ(五十音順に掲載)　(注1)
		#  $values[6]: 都道府県名　…………　漢字(コード順に掲載)　(注1,2)
		#  $values[7]: 市区町村名　…………　漢字(コード順に掲載)　(注1,2)
		#  $values[8]: 町域名　………………　漢字(五十音順に掲載)　(注1,2)

		my ($area_id, $city_id) = unpack("a2 a3", $values[0]);
		warn "No match area_name: $area_id\n" if $values[6] ne $AREA{$area_id};
		if ($values[8] eq '以下に掲載がない場合') {
			$values[8] = "";
			$values[5] = "";
		}
		$values[8] =~ s/[\（\(\）\)\、].*$// if $values[8] =~ /[\（\(\）\)\、]/;
		$values[5] =~ s/[\（\(\）\)\、].*$// if $values[5] =~ /[\（\(\）\)\、]/;
		$values[8] =~ s/^\d+.*$// if $values[8] =~ /^\d/;
		$values[5] =~ s/^\d+.*$// if $values[5] =~ /^\d/;
		$i++;
		unless ($seen{$values[2].':'.$values[6].':'.$values[7].':'.$values[8]}++) {
			$sth->bind_param(1, $i);
			$sth->bind_param(2, Encode::encode('utf8', $values[2]));
			$sth->bind_param(3, $area_id);
			$sth->bind_param(4, $city_id);
			$sth->bind_param(5, Encode::encode('utf8', $values[6]));
			$sth->bind_param(6, Encode::encode('utf8', $values[7]));
			$sth->bind_param(7, Encode::encode('utf8', $values[8]));
			$sth->bind_param(8, Encode::encode('utf8', $values[3]));
			$sth->bind_param(9, Encode::encode('utf8', $values[4]));
			$sth->bind_param(10, Encode::encode('utf8', $values[5]));
			$sth->execute;
			$j++;
		}
		# 一つの郵便番号に所属する住所の数をカウント(確認用)
		$postcode{$values[2]}++;
		if ($postcode{$values[2]} > $max_count) {
			$max_count = $postcode{$values[2]};
			$max_postid = $values[2];
		}
		progress_bar($i, $total) if $i == 1 || $i % 100 == 0;
	}
	progress_bar($i, $total);
	$sth->finish;
	$dbh->commit;

	my $opt_sql = $DATA_SOURCE =~ /:Pg:/ ? 'VACUUM ANALYZE' : 'OPTIMIZE TABLE zip';
	print "\n$opt_sql. Please wait...";
	$dbh->do($opt_sql);
	open STDERR, '>&', $stderr if $logging;
	print "\nCreated: $logfile\n";
	close $fh;

	print "Copied Postcode (total $j/$i items).\n";
	#print "Postcode $max_postid has $max_count addresses.\n";
}

sub fetch_ken_all {
	my $file = 'ken_all.lzh';
	my $url = 'http://www.post.japanpost.jp/zipcode/dl/kogaki/lzh/'.$file;
	my $dir = $C->{DIR_BACKUPS};
	my $lzhfile = $dir.'/'.$file;
	my $csvfile = $dir.'/ken_all.csv';
	my $csvFILE = $dir.'/KEN_ALL.CSV';
	if (-e $lzhfile) {
		unlink $lzhfile and print "Removed $lzhfile\n";
	}
	eval "use LWP::UserAgent;";
	$@ and die $@;
	my $ua = LWP::UserAgent->new;
	$ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows 98; Win 9x 4.90)");
	$ua->timeout(30);
	print "Downloading '$file' from $url\n";
	my $res;
	if ($LWP::UserAgent::VERSION >= 2.033) {
		my $total_size = $ua->head($url)->headers->content_length;
		my $fetch_size = 0;
		open my $fh, '>', $lzhfile or die "Can not open $lzhfile: $!";
		$res = $ua->get($url, ':content_cb' => sub {
							my ($chunk, $response, $protocol) = @_;
							print $fh $chunk;
							$fetch_size += length $chunk;
							progress_bar($fetch_size, $total_size);
						});
		close $fh;
	} else {
		print "Please wait...";
		$res = $ua->mirror($url, $lzhfile);
	}
	print "\n";
	$res->is_success or die "Error: $url";
	print "Saved as $lzhfile\n";
	if (-e $csvfile) {
		unlink $csvfile and print "Removed $csvfile\n";
	}
	!system "lha xw=$dir $lzhfile" or die $?;
	rename $csvFILE, $csvfile if -e $csvFILE;
}

sub progress_bar {
	my ($i, $total) = @_;
	my $rate = $i / $total;
	printf("%3s%%[%-61s] %6d/%d", int(100 * $rate), ("=" x int(60 * $rate)).'>', $i, $total);
	print "\r";
}

__DATA__
01,1,北海道,北海道・東北
02,1,青森県,北海道・東北
03,1,岩手県,北海道・東北
04,1,宮城県,北海道・東北
05,1,秋田県,北海道・東北
06,1,山形県,北海道・東北
07,1,福島県,北海道・東北
08,2,茨城県,関東
09,2,栃木県,関東
10,2,群馬県,関東
11,2,埼玉県,関東
12,2,千葉県,関東
13,2,東京都,関東
14,2,神奈川県,関東
15,3,新潟県,中部
16,3,富山県,中部
17,3,石川県,中部
18,3,福井県,中部
19,3,山梨県,中部
20,3,長野県,中部
21,3,岐阜県,中部
22,3,静岡県,中部
23,3,愛知県,中部
24,3,三重県,中部
25,4,滋賀県,近畿
26,4,京都府,近畿
27,4,大阪府,近畿
28,4,兵庫県,近畿
29,4,奈良県,近畿
30,4,和歌山県,近畿
31,5,鳥取県,中国
32,5,島根県,中国
33,5,岡山県,中国
34,5,広島県,中国
35,5,山口県,中国
36,5,徳島県,中国
37,5,香川県,中国
38,5,愛媛県,中国
39,5,高知県,中国
40,6,福岡県,九州・沖縄
41,6,佐賀県,九州・沖縄
42,6,長崎県,九州・沖縄
43,6,熊本県,九州・沖縄
44,6,大分県,九州・沖縄
45,6,宮崎県,九州・沖縄
46,6,鹿児島県,九州・沖縄
47,6,沖縄県,九州・沖縄
__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.

=head1 名称

 postal.pl - 郵便番号データのデータベース登録

=head1 用途

 ken_all.csv を元にデータベースに郵便番号データを登録します。

=head1 書式

 ./postal.pl

=head1 説明

 backups ディレクトリに ken_all.csv を用意して

 ./postal.pl

 を実行して下さい。

=head1 補足

 README も参照して下さい。

=cut
