#!/usr/bin/perl -w
# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

use FindBin;
use lib "$FindBin::Bin/plugins", "$FindBin::Bin/lib";
use strict;
use File::Find;
use File::Path qw(mkpath);
use ESP::SMS::Plugin 'load';
use ESP::SMS::Data;
use Cozy::Data::DBI::Schema;
$| = 1;

my $C = ESP::SMS::Plugin->config;
my $FIND_REGEXP = qr/^$/;
my $UMASK = $C->{UMASK};
my $DIR_LOCAL = $C->{DIR_LOCAL};
my $DIR_SKEL = $C->{DIR_SKEL} or die "No defined config parameter: DIR_SKEL\n";
my $DIR_TMPL = $C->{DIR_TMPL} or die "No defined config parameter: DIR_TMPL\n";
my $DIR_STATIC = $C->{DIR_STATIC} or die "No defined config parameter: DIR_STATIC\n";
my $DB_TYPE = $C->{DB_TYPE} or die "No defined parameter: DB_TYPE";
my $VERSION = shift or die "Usage: $0 (upgraded version number)";
my $dbh = ESP::SMS::Data->handler->dbh;
my $schema = Cozy::Data::DBI::Schema->new({ dbh => $dbh,
											modules => [],
											logfile => "$FindBin::Bin/schema.log",
											handler => 'Cozy::DataHandler::DBI::Schema::'.$DB_TYPE,
											engine => $DB_TYPE eq 'mysql' ? $C->{DB_ENGINE}||'MyISAM' : '',
											table => '',
											trace => 1 });

# 4.0(4.0.x含む)→4.1
if ($VERSION == 4.0) {
	# ●注意:jqueryのバージョンは変更されます。
	# Plugin.pm hrefの値に\$?追加、header,footer,nav,label,noscript

	# local/plugin/00config.plに以下追加して下さい。
	# 会員登録時のブラウザ認証
	# 本登録時に仮登録と本登録に使用したブラウザが同じかどうか確認して
	# 違う場合は本登録を拒否します。
	# ブラウザ認証を行わない場合は 0 に設定して下さい。
	# $C->{VERIFY_MEMBER_AGENT} = 1;

	# 検索語を更新するためにcron_d.plを実行して下さい。

	# local/skel/_302.null 空ファイル作成
	my $file = "$DIR_SKEL/_302.null";
	unless (-e $file) {
		if (open my $fh, '>', $file) {
			print "append $file\n";
			close $fh;
		} else {
			print "Can not append $file: $!\n";
		}
	}

	# モール用: 更新必須のstaticファイルのコピー
	do_sync([$DIR_SKEL.'/static/0/js/editor.js',
			 $DIR_SKEL.'/static/0/js/webmgr.js',
			 $DIR_SKEL.'/static/0/js/validator.js',
			 $DIR_SKEL.'/static/0/js/jquery.js'],
			qr/^$DIR_SKEL\/static\/0\/(.*)$/,
			[$DIR_TMPL.'/static/0/'], $UMASK, 1);

	# モール用: 新規追加分のstaticファイルのコピー
	do_sync($DIR_SKEL.'/static/0/',
			qr/^$DIR_SKEL\/static\/0\/(.*[a-zA-Z0-9_-]+\.(?:css|js|shtml|gif|jpg|png))$/,
			[$DIR_TMPL.'/static/0/'], $UMASK);

	# ショップ用: 新規追加分のstaticファイルのコピー
	my @dirs = map { "$DIR_TMPL/static/$_/" } get_shopid("$DIR_TMPL/static/");
	do_sync($DIR_SKEL.'/static/',
			qr/^$DIR_SKEL\/static\/([a-zA-Z0-9_-]+\.shtml)$/,
			\@dirs, $UMASK);

	# スマートフォン画面の表示/非表示項目の追加
	$schema->set_fields('ESP::SMS::Data::TRM', qw(trm_alias_type trm_alias_name trm_alias_open));
	do_schema('trm', '--install_data --update');
	# スマートフォン画面用の決済サービスの設定項目の追加
	$schema->set_fields('ESP::SMS::Data::PSV', qw(psv_tmpl2a));
	do_schema('psv', '--install_data --update --fields="psv_tmpl2a psv_name3 psv_desc3"');

	# サイトの更新
	do_manage('webmgr', 'build_all', 'exec=1 force=1 recursive=1');

	# アップグレードの場合はスマートフォン画面はデフォルトで非表示とする。
	do_sql('UPDATE trm SET trm_alias_open = 0 WHERE trm_id = 2');
}

sub do_sql {
	my ($sql) = @_;
	eval {
		$dbh->do($sql);
	};
	warn $@ if $@;
}

sub do_schema {
	my ($table, $opts) = @_;
	$table or die "ERROR: No parameter : table\n";
	my $command = "$DIR_LOCAL/schema.pl --table=$table";
	$command .= " $opts" if $opts;
	print $command, "\n";
	!system $command or die $?;
}

sub do_manage {
	my ($name, $pmd, $args) = @_;
	$name or die "ERROR: No parameter : name\n";
	$pmd or die "ERROR: No parameter : pmd\n";
	my $command = "$DIR_LOCAL/manage.pl name=$name pmd=$pmd";
	$command .= " $args" if $args;
	print $command, "\n";
	!system $command or die $?;
}

sub do_sync {
	my ($dir, $regexp, $todirs, $umask, $force) = @_;
	my @files;
	find({ wanted => sub {
			   my $file = $File::Find::name;
			   my ($path) = $file =~ /$regexp/ or return;
			   push @files, [$file, $path];
		   }, no_chdir => 1 }, ref $dir eq 'ARRAY' ? @$dir : $dir);
	for my $todir (@$todirs) {
		for my $ref (@files) {
			my ($file, $path) = @$ref;
			my $file2 = $todir.$path;
			if (-e $file2 && !$force) {
				print "skip: $file2\n";
				next;
			}
			my ($dir2) = $file2 =~ /^(.*)\/[^\/]*$/;
			unless (-d $dir2) {
				my $umask2 = umask();
				umask($umask);
				eval { mkpath($dir2) };
				umask($umask2);
				if ($@) {
					die "Couldn't create $dir2: $@";
				}
				print "make: $dir2\n";
			}
			my $umask2 = umask();
			umask($umask);
			open my $fh, '<', $file or die "Can not create file : $file\n";
			open my $fh2, '>', $file2 or die "Can not create file : $file2\n";
			while (<$fh>) {
				print $fh2 $_;
			}
			close $fh2;
			close $fh;
			umask($umask2);
			print "copy: $file\n";
			print "  to: $file2\n";
		}
	}
}

sub get_shopid {
	my ($dir) = @_;
	opendir my $sth, $dir or die "Can not open directory : $dir : $!";
	my @id;
	while (defined(my $name = readdir $sth)) {
		push @id, $name if $name =~ /^[0-9]+$/ && $name > 0;
	}
	closedir $sth;
	return sort { $a <=> $b } @id;
}

__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
