#!/usr/bin/perl -w
# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

use FindBin;
use lib "$FindBin::Bin/plugins", "$FindBin::Bin/lib";
use strict;
use ESP::SMS::Plugin 'load';
use ESP::SMS::Data;
use Cozy::Data::DBI::Schema;
use Getopt::Long;
$| = 1;

my $C = ESP::SMS::Plugin->config;
my @DB_TABLE = @{$C->{DB_TABLE}||[]} or die "No defined parameter: DB_TABLE";
my $DB_TYPE = $C->{DB_TYPE} or die "No defined parameter: DB_TYPE";
GetOptions(install => \my $install, install_data => \my $install_data, uninstall => \my $uninstall,
		   'output=s' => \my $output, update => \my $update, 'table=s' => \my $table,
		   'fields=s' => \my $fields);
if (!$install && !$install_data && !$uninstall && !$output) {
	print "Usage: $0 [--install|--install_data|--update|--uninstall|--output=(file path)|--table=(table)|--fields=(field1 field2 ...)]\n";
	exit 1;
}
my $schema = Cozy::Data::DBI::Schema->new({ dbh => ESP::SMS::Data->handler->dbh,
											modules => [ map { "ESP::SMS::Data::$_" } @DB_TABLE ],
											logfile => "$FindBin::Bin/schema.log",
											handler => 'Cozy::DataHandler::DBI::Schema::'.$DB_TYPE,
											engine => $DB_TYPE eq 'mysql' ? $C->{DB_ENGINE}||'InnoDB' : '',
											table => $table||'',
											fields => $fields ? [split(/[ ,]+/, $fields)] : [],
											trace => 1 });

$schema->update($update||0);
$schema->initdata(
[
 # 端末1
 {
  name => 'html',
  class => 'ESP::SMS::Data::TRM',
  where => [
			trm_id => 1
		   ],
  param => {
			trm_type => 'html',
			trm_name => 'PC',
			trm_alias_type => undef,
			trm_alias_name => undef,
			trm_alias_open => 0,
		   },
 },
 # 端末2
 {
  name => 'xhtml',
  class => 'ESP::SMS::Data::TRM',
  where => [
			trm_id => 2
		   ],
  param => {
			trm_type => 'xhtml',
			trm_name => '携帯',
			trm_alias_type => 'shtml',
			trm_alias_name => 'スマートフォン',
			trm_alias_open => 1,
		   },
 },
 # 設定1
 {
  name => 'ODR_METHOD',
  class => 'ESP::SMS::Data::CFG',
  where => [
			cfg_id => 'ODR_METHOD'
		   ],
  param => {
			cfg_status => 0,
			cfg_note => '決済方法',
			cfg_open => 1,
		   },
 },
 # 支払方法1
 {
  name => 'pay1',
  class => 'ESP::SMS::Data::PAY',
  where => [
			pay_id => 1
		   ],
  param => {
			pay_name => '銀行振込',
			pay_note => '',
		   },
 },
 # 支払方法2
 {
  name => 'pay2',
  class => 'ESP::SMS::Data::PAY',
  where => [
			pay_id => 2
		   ],
  param => {
			pay_name => '郵便振替',
			pay_note => '',
		   },
 },
 # 支払方法3
 {
  name => 'pay3',
  class => 'ESP::SMS::Data::PAY',
  where => [
			pay_id => 3
		   ],
  param => {
			pay_name => '代金引替',
			pay_note => '',
		   },
 },
 # 支払方法4
 {
  name => 'pay4',
  class => 'ESP::SMS::Data::PAY',
  where => [
			pay_id => 4
		   ],
  param => {
			pay_name => 'コンビニ決済',
			pay_note => '',
		   },
 },
 # 支払方法5
 {
  name => 'pay5',
  class => 'ESP::SMS::Data::PAY',
  where => [
			pay_id => 5
		   ],
  param => {
			pay_name => 'クレジット決済',
			pay_note => '',
		   },
 },
 # 支払方法6
 {
  name => 'pay6',
  class => 'ESP::SMS::Data::PAY',
  where => [
			pay_id => 6
		   ],
  param => {
			pay_name => '現金',
			pay_note => '',
		   },
 },
 # 配送方法1
 {
  name => 'dvy1',
  class => 'ESP::SMS::Data::DVY',
  where => [
			dvy_id => 1
		   ],
  param => {
			dvy_name => '宅配便',
			dvy_ticket => 0,
			dvy_open => 1,
		   },
 },
 # 配送方法5
 {
  name => 'dvy5',
  class => 'ESP::SMS::Data::DVY',
  where => [
			dvy_id => 5
		   ],
  param => {
			dvy_name => 'メールギフト',
			dvy_ticket => 1,
			dvy_open => 1,
		   },
 },
 # 配送方法10
 {
  name => 'dvy10',
  class => 'ESP::SMS::Data::DVY',
  where => [
			dvy_id => 10
		   ],
  param => {
			dvy_name => '店頭受取り',
			dvy_ticket => 0,
			dvy_open => 1,
		   },
 },
 # 都道府県(北海道)
 {
  name => 'are1',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 1
		   ],
  param => {
			are_name => '北海道',
			are_state => '北海道',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(青森県)
 {
  name => 'are2',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 2
		   ],
  param => {
			are_name => '青森県',
			are_state => '東北',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(岩手県)
 {
  name => 'are3',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 3
		   ],
  param => {
			are_name => '岩手県',
			are_state => '東北',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(宮城県)
 {
  name => 'are4',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 4
		   ],
  param => {
			are_name => '宮城県',
			are_state => '東北',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(秋田県)
 {
  name => 'are5',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 5
		   ],
  param => {
			are_name => '秋田県',
			are_state => '東北',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(山形県)
 {
  name => 'are6',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 6
		   ],
  param => {
			are_name => '山形県',
			are_state => '東北',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(福島県)
 {
  name => 'are7',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 7
		   ],
  param => {
			are_name => '福島県',
			are_state => '東北',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(茨城県)
 {
  name => 'are8',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 8
		   ],
  param => {
			are_name => '茨城県',
			are_state => '関東',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(栃木県)
 {
  name => 'are9',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 9
		   ],
  param => {
			are_name => '栃木県',
			are_state => '関東',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(群馬県)
 {
  name => 'are10',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 10
		   ],
  param => {
			are_name => '群馬県',
			are_state => '関東',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(埼玉県)
 {
  name => 'are11',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 11
		   ],
  param => {
			are_name => '埼玉県',
			are_state => '関東',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(千葉県)
 {
  name => 'are12',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 12
		   ],
  param => {
			are_name => '千葉県',
			are_state => '関東',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(東京都)
 {
  name => 'are13',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 13
		   ],
  param => {
			are_name => '東京都',
			are_state => '関東',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(神奈川県)
 {
  name => 'are14',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 14
		   ],
  param => {
			are_name => '神奈川県',
			are_state => '関東',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(新潟県)
 {
  name => 'are15',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 15
		   ],
  param => {
			are_name => '新潟県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(富山県)
 {
  name => 'are16',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 16
		   ],
  param => {
			are_name => '富山県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(石川県)
 {
  name => 'are17',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 17
		   ],
  param => {
			are_name => '石川県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(福井県)
 {
  name => 'are18',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 18
		   ],
  param => {
			are_name => '福井県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(山梨県)
 {
  name => 'are19',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 19
		   ],
  param => {
			are_name => '山梨県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(長野県)
 {
  name => 'are20',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 20
		   ],
  param => {
			are_name => '長野県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(岐阜県)
 {
  name => 'are21',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 21
		   ],
  param => {
			are_name => '岐阜県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(静岡県)
 {
  name => 'are22',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 22
		   ],
  param => {
			are_name => '静岡県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(愛知県)
 {
  name => 'are23',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 23
		   ],
  param => {
			are_name => '愛知県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(三重県)
 {
  name => 'are24',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 24
		   ],
  param => {
			are_name => '三重県',
			are_state => '中部',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(滋賀県)
 {
  name => 'are25',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 25
		   ],
  param => {
			are_name => '滋賀県',
			are_state => '近畿',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(京都府)
 {
  name => 'are26',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 26
		   ],
  param => {
			are_name => '京都府',
			are_state => '近畿',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(大阪府)
 {
  name => 'are27',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 27
		   ],
  param => {
			are_name => '大阪府',
			are_state => '近畿',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(兵庫県)
 {
  name => 'are28',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 28
		   ],
  param => {
			are_name => '兵庫県',
			are_state => '近畿',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(奈良県)
 {
  name => 'are29',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 29
		   ],
  param => {
			are_name => '奈良県',
			are_state => '近畿',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(和歌山県)
 {
  name => 'are30',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 30
		   ],
  param => {
			are_name => '和歌山県',
			are_state => '近畿',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(鳥取県)
 {
  name => 'are31',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 31
		   ],
  param => {
			are_name => '鳥取県',
			are_state => '中国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(島根県)
 {
  name => 'are32',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 32
		   ],
  param => {
			are_name => '島根県',
			are_state => '中国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(岡山県)
 {
  name => 'are33',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 33
		   ],
  param => {
			are_name => '岡山県',
			are_state => '中国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(広島県)
 {
  name => 'are34',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 34
		   ],
  param => {
			are_name => '広島県',
			are_state => '中国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(山口県)
 {
  name => 'are35',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 35
		   ],
  param => {
			are_name => '山口県',
			are_state => '中国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(徳島県)
 {
  name => 'are36',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 36
		   ],
  param => {
			are_name => '徳島県',
			are_state => '四国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(香川県)
 {
  name => 'are37',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 37
		   ],
  param => {
			are_name => '香川県',
			are_state => '四国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(愛媛県)
 {
  name => 'are38',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 38
		   ],
  param => {
			are_name => '愛媛県',
			are_state => '四国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(高知県)
 {
  name => 'are39',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 39
		   ],
  param => {
			are_name => '高知県',
			are_state => '四国',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(福岡県)
 {
  name => 'are40',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 40
		   ],
  param => {
			are_name => '福岡県',
			are_state => '九州',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(佐賀県)
 {
  name => 'are41',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 41
		   ],
  param => {
			are_name => '佐賀県',
			are_state => '九州',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(長崎県)
 {
  name => 'are42',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 42
		   ],
  param => {
			are_name => '長崎県',
			are_state => '九州',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(熊本県)
 {
  name => 'are43',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 43
		   ],
  param => {
			are_name => '熊本県',
			are_state => '九州',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(大分県)
 {
  name => 'are44',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 44
		   ],
  param => {
			are_name => '大分県',
			are_state => '九州',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(宮崎県)
 {
  name => 'are45',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 45
		   ],
  param => {
			are_name => '宮崎県',
			are_state => '九州',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(鹿児島県)
 {
  name => 'are46',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 46
		   ],
  param => {
			are_name => '鹿児島県',
			are_state => '九州',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(沖縄県)
 {
  name => 'are47',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 47
		   ],
  param => {
			are_name => '沖縄県',
			are_state => '沖縄',
			are_country => '日本',
			are_foreign => 0,
			are_open => 1,
		   },
 },
 # 都道府県(その他・海外中国)
 {
  name => 'are48',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 48
		   ],
  param => {
			are_name => '北京市',
			are_state => '海外(中国)',
			are_country => '中国',
			are_foreign => 1,
			are_open => 0,
		   },
 },
 # 都道府県(その他・海外中国)
 {
  name => 'are49',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 49
		   ],
  param => {
			are_name => '広東省',
			are_state => '海外(中国)',
			are_country => '中国',
			are_foreign => 1,
			are_open => 0,
		   },
 },
 # 都道府県(その他・海外アメリカ)
 {
  name => 'are50',
  class => 'ESP::SMS::Data::ARE',
  where => [
			are_id => 50
		   ],
  param => {
			are_name => 'ニューヨーク州',
			are_state => '海外(米国)',
			are_country => '米国',
			are_foreign => 1,
			are_open => 0,
		   },
 },
 # 趣味(インターネット)
 {
  name => 'anq1',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 1
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'インターネット',
		   },
 },
 # 趣味(アウトドア)
 {
  name => 'anq2',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 2
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'アウトドア',
		   },
 },
 # 趣味(コンピュータ)
 {
  name => 'anq3',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 3
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'コンピュータ',
		   },
 },
 # 趣味(つり)
 {
  name => 'anq4',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 4
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'つり',
		   },
 },
 # 趣味(ショッピング)
 {
  name => 'anq5',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 5
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'ショッピング',
		   },
 },
 # 趣味(野球)
 {
  name => 'anq6',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 6
		   ],
  param => {
			anq_oid => 1,
			anq_name => '野球',
		   },
 },
 # 趣味(ダイビング)
 {
  name => 'anq7',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 7
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'ダイビング',
		   },
 },
 # 趣味(ドライブ)
 {
  name => 'anq8',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 8
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'ドライブ',
		   },
 },
 # 趣味(ハングライダー)
 {
  name => 'anq9',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 9
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'ハングライダー',
		   },
 },
 # 趣味(カヌー)
 {
  name => 'anq10',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 10
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'カヌー',
		   },
 },
 # 趣味(ガーデニング)
 {
  name => 'anq11',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 11
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'ガーデニング',
		   },
 },
 # 趣味(カラオケ)
 {
  name => 'anq12',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 12
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'カラオケ',
		   },
 },
 # 趣味(テニス)
 {
  name => 'anq13',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 13
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'テニス',
		   },
 },
 # 趣味(映画鑑賞)
 {
  name => 'anq14',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 14
		   ],
  param => {
			anq_oid => 1,
			anq_name => '映画鑑賞',
		   },
 },
 # 趣味(音楽鑑賞)
 {
  name => 'anq15',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 15
		   ],
  param => {
			anq_oid => 1,
			anq_name => '音楽鑑賞',
		   },
 },
 # 趣味(旅行)
 {
  name => 'anq16',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 16
		   ],
  param => {
			anq_oid => 1,
			anq_name => '旅行',
		   },
 },
 # 趣味(懸賞)
 {
  name => 'anq17',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 17
		   ],
  param => {
			anq_oid => 1,
			anq_name => '懸賞',
		   },
 },
 # 趣味(スキー・スノーボード)
 {
  name => 'anq18',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 18
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'スキー・スノーボード',
		   },
 },
 # 趣味(ゴルフ)
 {
  name => 'anq19',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 19
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'ゴルフ',
		   },
 },
 # 趣味(サッカー)
 {
  name => 'anq20',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 20
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'サッカー',
		   },
 },
 # 趣味(バイク)
 {
  name => 'anq21',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 21
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'バイク',
		   },
 },
 # 趣味(パラグライダー)
 {
  name => 'anq22',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 22
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'パラグライダー',
		   },
 },
 # 趣味(山菜取り)
 {
  name => 'anq23',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 23
		   ],
  param => {
			anq_oid => 1,
			anq_name => '山菜取り',
		   },
 },
 # 趣味(竹の子取り)
 {
  name => 'anq24',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 24
		   ],
  param => {
			anq_oid => 1,
			anq_name => '竹の子取り',
		   },
 },
 # 趣味(キノコ取り)
 {
  name => 'anq25',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 25
		   ],
  param => {
			anq_oid => 1,
			anq_name => 'キノコ取り',
		   },
 },
 # 趣味(山歩き)
 {
  name => 'anq26',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 26
		   ],
  param => {
			anq_oid => 1,
			anq_name => '山歩き',
		   },
 },
 # 好み(住宅)
 {
  name => 'anq27',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 27
		   ],
  param => {
			anq_oid => 2,
			anq_name => '住宅',
		   },
 },
 # 好み(マネー・不動産)
 {
  name => 'anq28',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 28
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'マネー・不動産',
		   },
 },
 # 好み(料理)
 {
  name => 'anq29',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 29
		   ],
  param => {
			anq_oid => 2,
			anq_name => '料理',
		   },
 },
 # 好み(ファッション)
 {
  name => 'anq30',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 30
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'ファッション',
		   },
 },
 # 好み(花)
 {
  name => 'anq31',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 31
		   ],
  param => {
			anq_oid => 2,
			anq_name => '花',
		   },
 },
 # 好み(化粧品)
 {
  name => 'anq32',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 32
		   ],
  param => {
			anq_oid => 2,
			anq_name => '化粧品',
		   },
 },
 # 好み(自然食品)
 {
  name => 'anq33',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 33
		   ],
  param => {
			anq_oid => 2,
			anq_name => '自然食品',
		   },
 },
 # 好み(パソコン)
 {
  name => 'anq34',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 34
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'パソコン',
		   },
 },
 # 好み(書籍・コミック)
 {
  name => 'anq35',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 35
		   ],
  param => {
			anq_oid => 2,
			anq_name => '書籍・コミック',
		   },
 },
 # 好み(ゲーム)
 {
  name => 'anq36',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 36
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'ゲーム',
		   },
 },
 # 好み(スクール・資格)
 {
  name => 'anq37',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 37
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'スクール・資格',
		   },
 },
 # 好み(転職)
 {
  name => 'anq38',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 38
		   ],
  param => {
			anq_oid => 2,
			anq_name => '転職',
		   },
 },
 # 好み(生活空間)
 {
  name => 'anq39',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 39
		   ],
  param => {
			anq_oid => 2,
			anq_name => '生活空間',
		   },
 },
 # 好み(カーライフ)
 {
  name => 'anq40',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 40
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'カーライフ',
		   },
 },
 # 好み(スポーツ)
 {
  name => 'anq41',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 41
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'スポーツ',
		   },
 },
 # 好み(占い)
 {
  name => 'anq42',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 42
		   ],
  param => {
			anq_oid => 2,
			anq_name => '占い',
		   },
 },
 # 好み(インターネットサービス)
 {
  name => 'anq43',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 43
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'インターネットサービス',
		   },
 },
 # 好み(ニュース・イベント)
 {
  name => 'anq44',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 44
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'ニュース・イベント',
		   },
 },
 # 好み(グルメ)
 {
  name => 'anq45',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 45
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'グルメ',
		   },
 },
 # 好み(健康)
 {
  name => 'anq46',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 46
		   ],
  param => {
			anq_oid => 2,
			anq_name => '健康',
		   },
 },
 # 好み(ビジネス)
 {
  name => 'anq47',
  class => 'ESP::SMS::Data::ANQ',
  where => [
			anq_id => 47
		   ],
  param => {
			anq_oid => 2,
			anq_name => 'ビジネス',
		   },
 },
 # モール情報
 {
  name => 'shop',
  class => 'ESP::SMS::Data::SHP',
  where => [
			shp_id => 0
		   ],
  param => {
			shp_id => 0,
			shp_lid => 'ja',
			shp_name => $C->{site_name},
			shp_kana => $C->{site_name},
			shp_admin => 'モール管理者',
			shp_dealer => '株式会社モール',
			shp_email => $C->{EMAIL},
			shp_zip => 1030028,
			shp_are_id => 13,
			shp_are_name => '東京都',
			shp_city => '中央区',
			shp_town => '八重洲1丁目1番',
			shp_tel => '03-0000-0000',
			shp_fax => '03-0000-0001',
			shp_robots => 'index,follow',
			shp_keywords => 'ネットショッピング,通信販売,通販',
			shp_description => '総合ショッピングモールです。',
			shp_author => $C->{site_name},
			shp_copyright => 'Copyright(C) Mall',
			shp_dir => 'static',
			shp_passwd => 'e3056168f492d042422b82c9bc2dca23',
			shp_salt => '12345',
			shp_url => $C->{URL_STATIC},
			shp_edit => 1,
			shp_build => 1,
			shp_open => 1,
			shp_alive => 1,
		   },
 },
 # 決済サービス(ZEUS)
 {
  name => 'payment_service1_html',
  class => 'ESP::SMS::Data::PSV',
  where => [
			psv_page => 'zeus',
		   ],
  param => {
			psv_name => 'ZEUS クレジット決済',
			psv_dealer => '株式会社ゼウス',
			psv_url => 'http://www.cardservice.co.jp/service/connection/linkpoint.html',
			psv_open => 1,
			psv_note => <<'EOF',
<% LOAD .URL -%>
<ul class="desc">
<li>ゼウス決済サービス(リンク型)に対応しています。</li>
<li>PC画面は Link Point CGI、携帯(スマートフォン含む)画面は Link Point Mobile(SSL決済画面)を使用します。</li>
<li>ご利用にはゼウス決済サービス(リンク型)のご契約が必要です。詳しくはゼウス様にお問い合わせ下さい。</li>
</ul>
<h5>クレジット決済を利用する場合の設定手順</h5>
<ol class="desc">
<li>この画面の「決済サービスの利用」を使用するに設定して下さい。</li>
<li>この画面の「クライアントIP」の欄にゼウスから取得したコードを設定して下さい。</li>
<li>この画面の「受信許可IPアドレス」の欄に接続資料(リンク型)に記載のCGIコールの送信元「IPアドレス」を設定して下さい。</li>
<li>この画面の「受信許可エージェント名」の欄に接続資料(リンク型)に記載のCGIコールの送信元「エージェント名」を設定して下さい。</li>
<li>この画面の「登録確認画面へ」ボタンを押して登録を確定して設定を更新して下さい。</li>
<li>次にショップ管理室の[ホーム]-[決済方法]-[新規決済方法を追加する]から以下を選択して登録して下さい。<br />
<table class="innerbox">
<tr><th>支払方法</th><td>「クレジット決済」</td></tr>
<tr><th>決済方法の利用</th><td>「使用する」</td></tr>
<tr><th>決済サービスの利用</th><td>「ZEUS クレジット決済」</td></tr>
</table>
※上記以外の項目は用途に応じて設定して下さい。<br />
※既にクレジット決済を使用した決済方法が登録されている場合は、新規登録せずに[編集]から「決済サービスの利用」のみを設定して下さい。</li>
<li>続いてゼウスの決済状況確認画面にログインして、CGIコールの送信先を以下のように設定して下さい。PC、携帯(スマートフォン含む)ともに共通です。<br />
<textarea name="sample" rows="2" cols="60" class="str">
<% cgi_order.https %>/<% shp_id %>/zeus/reply.html?sendid=&lt;ID&gt;&amp;money=&lt;KING&gt;&amp;result=OK
</textarea>
</li>
<li>最後に実際に商品を購入して、クレジット決済を選択し、テストカード番号で決済が正しく行われるか確認して下さい。</li>
<li>決済が完了した場合は、モールシステムから決済完了のメールが届きます。また管理室の[注文管理]で注文情報の支払が「済」に設定されます。</li>
<li>動作しない場合は、もう一度設定をご確認下さい。またモール管理者に「システム情報のログにエラーが発生していないか」をお問い合わせ下さい。</li>
</ol>
<h5>クレジット決済を無効にする場合の設定手順</h5>
<ol class="desc">
<li>ショップ管理室の[ホーム]-[決済方法]-[クレジット決済]の編集から「決済方法の利用」を使用しないに設定して登録して下さい。</li>
<li>この画面の「決済サービスの利用」を使用しないに設定して下さい。</li>
<li>この画面の「登録確認画面へ」ボタンを押して登録を確定して設定を更新して下さい。</li>
</ol>
EOF
			psv_tmpl1 => <<'EOF',
<% LOAD .URL -%>
<form action="https://linkpt.cardservice.co.jp/cgi-bin/order.cgi?orders" method="post" name="zeus" target="_blank" accept-charset="Shift_JIS" id="zeus">
<input type="hidden" name="clientip" value="<% psd_key1 %>" />
<input type="hidden" name="send" value="mall" />
<input type="hidden" name="sendid" value="<% bil_id %>" />
<input type="hidden" name="usrname" value="<% IF adr_alpha2 %><% adr_alpha2 %><% IF adr_alpha1 %> <% adr_alpha1 %><% END %><% END %>" />
<input type="hidden" name="usrmail" value="<% bil_email %>" />
<input type="hidden" name="usrtel" value="<% adr_tel.number %>" />
<input type="hidden" name="custom" value="yes" />
<input type="hidden" name="money" value="<% bil_price %>" />
<input type="submit" name="submit" value="決済ページに行く" class="submit" />
</form>
EOF
			psv_tmpl2 => <<'EOF',
<% LOAD .URL -%>
<form method="post" action="https://linkpt.cardservice.co.jp/cgi-bin/order.cgi" target="_blank">
<input type="hidden" name="clientip" value="<% psd_key2 %>" />
<input type="hidden" name="send" value="mall" />
<input type="hidden" name="act" value="imode" />
<input type="hidden" name="sendid" value="<% bil_id %>" />
<input type="hidden" name="usrname" value="<% IF adr_alpha2 %><% adr_alpha2 %><% IF adr_alpha1 %> <% adr_alpha1 %><% END %><% END %>" />
<input type="hidden" name="usrtel" value="<% adr_tel.number %>" />
<input type="hidden" name="usrmail" value="<% bil_email %>" />
<input type="hidden" name="money" value="<% bil_price %>" />
<input type="hidden" name="siteurl" value="<% cgi_static %>/index.xhtml?rid=<% rid %>" />
<input type="hidden" name="sitestr" value="<% site_name %>" />
<input type="submit" name="submit" value="決済ページに行く" style="font-size:x-small;" />
</form>
EOF
			psv_tmpl2a => <<'EOF',
<% LOAD .URL -%>
<form action="https://linkpt.cardservice.co.jp/cgi-bin/credit/order.cgi" method="post" data-ajax="false" name="zeus" t
arget="_blank" accept-charset="Shift_JIS" id="zeus">
<input type="hidden" name="clientip" value="<% psd_key3 %>" />
<input type="hidden" name="send" value="mall" />
<input type="hidden" name="sendid" value="<% bil_id %>" />
<input type="hidden" name="usrname" value="<% IF adr_alpha2 %><% adr_alpha2 %><% IF adr_alpha1 %> <% adr_alpha1 %><% END %><% END %>" />
<input type="hidden" name="email" value="<% bil_email %>" />
<input type="hidden" name="telno" value="<% adr_tel.number %>" />
<input type="hidden" name="custom" value="yes" />
<input type="hidden" name="money" value="<% bil_price %>" />
<input type="submit" name="submit" value="決済ページに行く" data-theme="e" data-icon="arrow-r" />
</form>
EOF
			psv_name1 => 'クライアントIP(PCサイト用)',
			psv_desc1 => 'クライアントIP(PCサイト用)を記入して下さい。',
			psv_name2 => 'クライアントIP(携帯用)',
			psv_desc2 => 'クライアントIP(携帯用)を記入して下さい。',
			psv_name3 => 'クライアントIP(スマートフォン用)',
			psv_desc3 => 'クライアントIP(スマートフォン用)を記入して下さい。',
			psv_reply => 1,
			psv_bilkey => 'sendid',
			psv_prikey => 'money',
			psv_iftmpl => <<'EOF',
<% IF result.eq(OK) %>OK<% ELSE %>NG<% END %>
EOF
			psv_retmpl => <<'EOF',
Credit: OK
EOF
			psv_totmpl => <<'EOF',
result=<% result %>
EOF
			psv_raddr => '',
			psv_agent => '',
		   },
 },
 # 決済サービス(クロネコ@ペイメント)PC専用
 {
  name => 'payment_service2_html',
  class => 'ESP::SMS::Data::PSV',
  where => [
			psv_page => 'kuroneko',
		   ],
  param => {
			psv_name => 'クロネコ＠ペイメント(PC画面のみ)',
			psv_dealer => 'ヤマトフィナンシャル株式会社',
			psv_url => 'https://payment.kuronekoyamato.co.jp/help/hanbai/index.html',
			psv_open => 1,
			psv_note => <<'EOF',
<% LOAD .URL -%>
<ul class="desc">
<li>クロネコ＠ペイメント(リンク方式)に対応しています。</li>
<li>ご利用にはクロネコ＠ペイメントのご契約が必要です。詳しくはヤマトフィナンシャル様にお問い合わせ下さい。</li>
<li>決済結果は通知されませんので、入金を確認した後は手動で注文情報の支払を「済」に設定して下さい。</li>
</ul>
<h5>クレジット決済を利用する場合の設定手順</h5>
<ol class="desc">
<li>この画面の「決済サービスの利用」を使用するに設定して下さい。</li>
<li>この画面の「加盟店コード」の欄にクロネコ＠ペイメントから取得したコードを設定して下さい。</li>
<li>この画面の「動作モード」の欄に L_TEST を設定して下さい。</li>
<li>この画面の「登録確認画面へ」ボタンを押して登録を確定して設定を更新して下さい。</li>
<li>次にショップ管理室の[ホーム]-[決済方法]-[新規決済方法を追加する]から以下を選択して登録して下さい。<br />
<table class="innerbox">
<tr><th>支払方法</th><td>「クレジット決済」</td></tr>
<tr><th>決済方法の利用</th><td>「使用する」</td></tr>
<tr><th>決済サービスの利用</th><td>「クロネコ＠ペイメント(PC画面のみ)」</td></tr>
</table>
※上記以外の項目は用途に応じて設定して下さい。<br />
※既にクレジット決済を使用した決済方法が登録されている場合は、新規登録せずに[編集]から「決済サービスの利用」のみを設定して下さい。</li>
<li>最後に実際に商品を購入して、クレジット決済を選択し、テストカード番号で決済が正しく行われるか確認して下さい。</li>
<li>決済が完了した場合は、クロネコ＠ペイメントから決済完了のメールが届きます。</li>
<li>動作しない場合は、もう一度設定をご確認下さい。またモール管理者に「システム情報のログにエラーが発生していないか」をお問い合わせ下さい。</li>
<li>動作確認で問題なければこの画面の「動作モード」の値を L_TEST から V_W02 に変更して登録して下さい。</li>
</ol>
<h5>クレジット決済を無効にする場合の設定手順</h5>
<ol class="desc">
<li>ショップ管理室の[ホーム]-[決済方法]-[クレジット決済]の編集から「決済方法の利用」を使用しないに設定して登録して下さい。</li>
<li>この画面の「決済サービスの利用」を使用しないに設定して下さい。</li>
<li>この画面の「登録確認画面へ」ボタンを押して登録を確定して設定を更新して下さい。</li>
</ol>
EOF
			psv_tmpl1 => <<'EOF',
<% LOAD .URL -%>
<form action="https://payment.kuronekoyamato.co.jp/kuroneko/servlet/YCS_ServletC" method="post" name="kuroneko" target="_blank" accept-charset="Shift_JIS" id="kuroneko">
<input type="hidden" name="TRS_MAP" value="<% psd_key2 %>" />
<input type="hidden" name="trader_code" value="<% psd_key1 %>" />
<input type="hidden" name="order_no" value="<% bil_id %>" />
<input type="hidden" name="goods_name" value="<% shp_name %>お買上げ商品" />
<input type="hidden" name="settle_price" value="<% bil_price %>" />
<input type="hidden" name="buyer_name_kanji" value="<% adr_name1.hk2z %><% adr_name2.hk2z %>" />
<input type="hidden" name="buyer_tel" value="<% adr_tel.number %>" />
<input type="hidden" name="buyer_email" value="<% bil_email %>" />
<input type="submit" name="submit" value="決済ページに行く" class="submit" />
</form>
EOF
			psv_name1 => '加盟店コード',
			psv_desc1 => '加盟店コード(半角数字)を記入して下さい。',
			psv_name2 => '動作モード',
			psv_desc2 => 'テスト環境に接続する場合は「L_TEST」、本番環境に接続する場合は「V_W02」を記入して下さい。決済のテストを行う場合はテスト環境に接続(L_TESTを記入)して下さい。',
			psv_reply => 0,
			psv_bilkey => '',
			psv_prikey => '',
			psv_iftmpl => '',
			psv_retmpl => '',
			psv_raddr => '',
			psv_agent => '',
		   },
 },
 # 決済サービス(ルミーズ)
 {
  name => 'payment_service3_html',
  class => 'ESP::SMS::Data::PSV',
  where => [
			psv_page => 'remise',
		   ],
  param => {
			psv_name => 'ルミーズ クレジット決済',
			psv_dealer => 'ルミーズ株式会社',
			psv_url => 'http://www.remise.jp/',
			psv_open => 1,
			psv_note => <<'EOF',
<% LOAD .URL -%>
<ul class="desc">
<li>クレジットカード決済(PC環境・携帯(スマートフォン含む)環境の両方)にのみ対応しています。(コンビニ決済には対応しておりません。)</li>
<li>ご利用にはルミーズ決済サービスのご契約が必要です。詳しくはルミーズ様にお問い合わせ下さい。</li>
</ul>
<h5>クレジット決済を利用する場合の設定手順</h5>
<ol class="desc">
<li>この画面の「決済サービスの利用」を使用するに設定して下さい。</li>
<li>この画面の「店コード」と「ホスト番号」の欄にルミーズから取得したコード(半角英数字)を設定して下さい。</li>
<li>この画面の「動作モード」の欄に0を設定して下さい。</li>
<li>この画面の「登録確認画面へ」ボタンを押して登録を確定して設定を更新して下さい。</li>
<li>次にショップ管理室の[ホーム]-[決済方法]-[新規決済方法を追加する]から以下を選択して登録して下さい。<br />
<table class="innerbox">
<tr><th>支払方法</th><td>「クレジット決済」</td></tr>
<tr><th>決済方法の利用</th><td>「使用する」</td></tr>
<tr><th>決済サービスの利用</th><td>「ルミーズ クレジット決済」</td></tr>
</table>
※上記以外の項目は用途に応じて設定して下さい。<br />
※既にクレジット決済を使用した決済方法が登録されている場合は、新規登録せずに[編集]から「決済サービスの利用」のみを設定して下さい。</li>
<li>続いてルミーズの管理画面(加盟店バックヤードシステム)にログインし、「各種設定」から「ホスト設定」を選択して、「結果通知URL」を以下のように設定して下さい。<br />
<textarea name="sample" rows="2" cols="60" class="str">
<% cgi_order.https %>/<% shp_id %>/remise/reply.html
</textarea>
</li>
<li>最後に実際に商品を購入して、クレジット決済を選択し、テストカード番号で決済が正しく行われるか確認して下さい。</li>
<li>決済が完了した場合は、モールシステムから決済完了のメールが届きます。また管理室の[注文管理]で注文情報の支払が「済」に設定されます。</li>
<li>動作しない場合は、もう一度設定をご確認下さい。またモール管理者に「システム情報のログにエラーが発生していないか」をお問い合わせ下さい。</li>
<li>動作確認で問題なければこの画面の「動作モード」の値を 0 から 1 に変更して登録して下さい。</li>
</ol>
<h5>クレジット決済を無効にする場合の設定手順</h5>
<ol class="desc">
<li>ショップ管理室の[ホーム]-[決済方法]-[クレジット決済]の編集から「決済方法の利用」を使用しないに設定して登録して下さい。</li>
<li>この画面の「決済サービスの利用」を使用しないに設定して下さい。</li>
<li>この画面の「登録確認画面へ」ボタンを押して登録を確定して設定を更新して下さい。</li>
</ol>
EOF
			psv_tmpl1 => <<'EOF',
<% LOAD .URL -%>
<% IF psd_key3 %>
<%# ↓本番環境用の接続先(https://test.remise.jp/〜の部分を本番環境用の「決済情報送信先URL」に変更して下さい。) -%>
<form action="https://test.remise.jp/rpgw2/pc/card/paycard.aspx" method="post" name="remise" target="_blank" accept-charset="Shift_JIS" id="remise">
<% ELSE -%>
<%# ↓テスト環境用の接続先 -%>
<form action="https://test.remise.jp/rpgw2/pc/card/paycard.aspx" method="post" name="remise" target="_blank" accept-charset="Shift_JIS" id="remise">
<% END -%>
<input type="hidden" name="S_TORIHIKI_NO" value="<% bil_id %>" />
<input type="hidden" name="MAIL" value="" />
<input type="hidden" name="AMOUNT" value="<% bil_price.without_tax(bil_price) %>" />
<input type="hidden" name="TAX" value="<% bil_price.tax(bil_price) %>" />
<input type="hidden" name="TOTAL" value="<% bil_price %>" />
<input type="hidden" name="SHOPCO" value="<% psd_key1 %>" />
<input type="hidden" name="HOSTID" value="<% psd_key2 %>" />
<input type="hidden" name="JOB" value="CAPTURE" />
<input type="hidden" name="ITEM" value="0000120" />
<input type="hidden" name="METHOD" value="10" />
<input type="hidden" name="EXITURL" value="<% cgi_order %>/0/payment.html" />
<input type="submit" name="submit" value="決済ページに行く" class="submit" />
</form>
EOF
			psv_tmpl2 => <<'EOF',
<% LOAD .URL -%>
<% IF psd_key3 %>
<%# ↓本番環境用の接続先(https://test.remise.jp/〜の部分を本番環境用の「決済情報送信先URL」に変更して下さい。) -%>
<form method="post" action="https://test.remise.jp/rpgw2/mbl/card/paycard.aspx" target="_blank">
<% ELSE -%>
<%# ↓テスト環境用の接続先 -%>
<form method="post" action="https://test.remise.jp/rpgw2/mbl/card/paycard.aspx" target="_blank">
<% END -%>
<input type="hidden" name="S_TORIHIKI_NO" value="<% bil_id %>" />
<input type="hidden" name="MAIL" value="" />
<input type="hidden" name="AMOUNT" value="<% bil_price.without_tax(bil_price) %>" />
<input type="hidden" name="TAX" value="<% bil_price.tax(bil_price) %>" />
<input type="hidden" name="TOTAL" value="<% bil_price %>" />
<input type="hidden" name="SHOPCO" value="<% psd_key1 %>" />
<input type="hidden" name="HOSTID" value="<% psd_key2 %>" />
<input type="hidden" name="JOB" value="CAPTURE" />
<input type="hidden" name="ITEM" value="0000120" />
<input type="hidden" name="METHOD" value="10" />
<input type="hidden" name="EXITURL" value="<% cgi_order %>/0/payment.xhtml?rid=<% rid %>" />
<input type="submit" name="submit" value="決済ページに行く" style="font-size:x-small;" />
</form>
EOF
			psv_tmpl2a => <<'EOF',
<% LOAD .URL -%>
<% IF psd_key3 %>
<%# ↓本番環境用の接続先(https://test.remise.jp/〜の部分を本番環境用の「決済情報送信先URL」に変更して下さい。) -%>
<form action="https://test.remise.jp/rpgw2/pc/card/paycard.aspx" method="post" name="remise" target="_blank" accept-charset="Shift_JIS" id="remise">
<% ELSE -%>
<%# ↓テスト環境用の接続先 -%>
<form action="https://test.remise.jp/rpgw2/pc/card/paycard.aspx" method="post" data-ajax="false" name="remise" target="_blank" accept-charset="Shift_JIS" id="remise">
<% END -%>
<input type="hidden" name="S_TORIHIKI_NO" value="<% bil_id %>" />
<input type="hidden" name="MAIL" value="" />
<input type="hidden" name="AMOUNT" value="<% bil_price.without_tax(bil_price) %>" />
<input type="hidden" name="TAX" value="<% bil_price.tax(bil_price) %>" />
<input type="hidden" name="TOTAL" value="<% bil_price %>" />
<input type="hidden" name="SHOPCO" value="<% psd_key1 %>" />
<input type="hidden" name="HOSTID" value="<% psd_key2 %>" />
<input type="hidden" name="JOB" value="CAPTURE" />
<input type="hidden" name="ITEM" value="0000120" />
<input type="hidden" name="METHOD" value="10" />
<input type="hidden" name="EXITURL" value="<% cgi_order %>/0/payment.shtml" />
<input type="submit" name="submit" value="決済ページに行く" data-theme="e" data-icon="arrow-r" />
</form>
EOF
			psv_name1 => '店コード',
			psv_desc1 => '店コード(半角英数字)を記入して下さい。(店コード例: RMS00000)',
			psv_name2 => 'ホスト番号',
			psv_desc2 => 'ホスト番号(半角英数字)を記入して下さい。(ホスト番号例: 00000000)',
			psv_name3 => '動作モード',
			psv_desc3 => 'テスト環境に接続する場合は「0」、本番環境に接続する場合は「1」を記入して下さい。決済のテストを行う場合はテスト環境に接続(0を記入)して下さい。',
			psv_reply => 1,
			psv_bilkey => 'X-S_TORIHIKI_NO',
			psv_prikey => 'X-TOTAL',
			psv_iftmpl => <<'EOF',
<% IF X_R_CODE.eq(0:0000) %><% IF X_ERRLEVEL.eq(0) %>OK<% ELSE %>NG<% END %><% ELSE %>NG<% END %>
EOF
			psv_retmpl => <<'EOF',
<SDBKDATA>STATUS=<% IF CARIER_TYPE %>900<% ELSE %>800<% END %></SDBKDATA>
EOF
			psv_totmpl => <<'EOF',
X-R_CODE=<% X_R_CODE %>
X-ERRLEVEL=<% X_ERRLEVEL %>
CARIER_TYPE=<% CARIER_TYPE %>
EOF
			psv_raddr => '',
			psv_agent => '',
		   },
 },
 # PC用注文確認メール
 {
  name => 'order_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'order',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 1,
			cml_open => 1,
			cml_name => '注文確認',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_reply => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>ご注文確認/自動配信メール(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

ご注文頂きありがとうございます。

[注文番号] <% order_id %>
[注文日時] <% datetime.timef(%04d/%02d/%02d %02d:%02d:%02d) %>
[支払方法] <% bil_pay_name %>
[配送方法] <% bil_dvy_name %>
[お届け先] <% OrderSell.size %>件
[会計先名] <% shp_name %>
[通信欄]
<% bil_req1 %>

ご連絡先
------------------------------------------------------------------------
[お名前] <% adr_name1 %> <% adr_name2 %> (<% adr_kana1 %> <% adr_kana2 %>) 様
[郵便番号] <% adr_zip.zipcode %>
[住所]     <% adr_are_name %><% adr_city %>
           <% adr_town %><% adr_etc %>
[電話番号] <% adr_tel %>
[FAX番号]  <% adr_fax %>
[メールアドレス] <% bil_email %>

<% IF slt_note2 -%>
<% pay_name %>/<% dvy_name %>について
------------------------------------------------------------------------
<% slt_note2 %>
<% ELSE -%>
<% IF slt_note1 -%>
<% pay_name %>/<% dvy_name %>について
------------------------------------------------------------------------
<% slt_note1 %>
<% END -%>
<% END -%>

ご注文商品
------------------------------------------------------------------------
<% FOR OrderItems -%>
[商品登録コード: <% item_id %>]
<% gds_name %> <% gds_options %>
@<% gds_bprice %> x <% odr_number %> = <% odr_bprice %>(円)
<% END -%>
------------------------------------------------------------------------

<% FOR OrderSell -%>
お届け先(<% :count %>)
------------------------------------------------------------------------
<% IF sel_ticket -%>
[お名前] <% sel_to %> 様
[メールアドレス] <% sel_email %>
[タイトル] <% sel_subject %>
[メッセージ]
<% sel_message %>
<% IF $slt_gift -%>
[ラッピングサービス] <% IF sel_gift %>利用する<% ELSE %>利用しない<% END %>
<% IF sel_gift %>[ラッピングメッセージ]
<% sel_req2 %>
<% END -%>
<% END -%>
------------------------------------------------------------------------
商品小計                <% sel_bprice %>(円)
地域別送料(別)
離島追加送料(別)
商品追加送料            <% sel_acharge %>(円)
クール便料(別)
ラッピング料(別)
小計                    <% sel_price %>(円)
------------------------------------------------------------------------
<% ELSE -%>
[お名前] <% adr_name1 %> <% adr_name2 %> (<% adr_kana1 %> <% adr_kana2 %>) 様
[郵便番号] <% adr_zip.zipcode %>
[住所]     <% adr_are_name %><% adr_city %>
           <% adr_town %><% adr_etc %>
[電話番号] <% adr_tel %>
[FAX番号]  <% adr_fax %>
[時間帯指定] <% sel_req1 %>
<% IF $slt_gift -%>
[ラッピングサービス] <% IF sel_gift %>利用する<% ELSE %>利用しない<% END %>
<% IF sel_gift %>[ラッピングメッセージ]
<% sel_req2 %>
<% END -%>
<% END -%>
[送料区分] <% sel_dvy_name %>/<% adr_are_name %>/数量<% sel_number %>/重量<% sel_weight %>/容量<% sel_volume %>
------------------------------------------------------------------------
商品小計                <% sel_bprice %>(円)
地域別送料              <% sel_pcharge %>(円)
離島追加送料            <% sel_icharge %>(円)
商品追加送料            <% sel_acharge %>(円)
クール便料              <% sel_ccharge %>(円)
ラッピング料            <% sel_gcharge %>(円)
小計                    <% sel_price %>(円)
------------------------------------------------------------------------
<% END -%>
<% END -%>

お支払い
------------------------------------------------------------------------
商品合計                <% bil_bprice %>(円) (内税 <% bil_tprice %>円)
<% IF dvy_ticket -%>
地域別送料合計(別)
離島追加送料合計(別)
商品追加送料合計            <% bil_acharge %>(円)
クール便料合計(別)
ラッピング料合計(別)
決済手数料              <% bil_fcharge %>(円)
ポイント割引            <% IF bil_pprice %>-<% bil_pprice %><% ELSE %>0<% END %>(円)
------------------------------------------------------------------------
お支払い合計            <% bil_price %>(円) (送料・手数料別)
※送料・手数料込みの金額は、ご注文承認後にお知らせいたします。
------------------------------------------------------------------------
<% ELSE -%>
地域別送料合計          <% bil_pcharge %>(円)
離島追加送料合計        <% bil_icharge %>(円)
商品追加送料合計        <% bil_acharge %>(円)
クール便料合計          <% bil_ccharge %>(円)
ラッピング料合計        <% bil_gcharge %>(円)
決済手数料              <% bil_fcharge %>(円)
ポイント割引            <% IF bil_pprice %>-<% bil_pprice %><% ELSE %>0<% END %>(円)
------------------------------------------------------------------------
お支払い合計            <% bil_price %>(円)
------------------------------------------------------------------------
<% END -%>

ご注文内容・注文履歴は以下のホームページでもご確認頂けます。
<% cgi_customer %>/0/history.html

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
定休日:<% shp_holiday %>
<% shp_url %>/
メールアドレス:<% shp_email %>
------------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # 携帯用注文確認メール
 {
  name => 'order_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'order',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 1,
			cml_open => 1,
			cml_name => '注文確認',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_reply => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>ご注文確認/自動配信メール(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

ご注文頂きありがとうございます。

●ご注文内容
[注文日時]
<% datetime.timef(%04d/%02d/%02d %02d:%02d:%02d) %>
[注文番号]
<% order_id %>
[通信欄]
<% bil_req1 %>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.xhtml

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
定休日:<% shp_holiday %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
<% REMOTE_HOST %>
<% REMOTE_ADDR %>
<% HTTP_REFERER %>
<% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # PC用注文受付メール
 {
  name => 'confirm_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'confirm',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 2,
			cml_open => 1,
			cml_name => '注文受付',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>ご注文の受付確認(<% order_id %>)',
									 eml_body =>  <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

ご注文頂きありがとうございます。
ご注文を受付けいたしました。お支払い内容をご確認下さい。

[注文番号] <% order_id %>
[注文日時] <% bil_create.timef(%04d/%02d/%02d %02d:%02d) %>
[支払方法] <% bil_pay_name %>
[配送方法] <% bil_dvy_name %>
[お届け先] <% OrderSell.size %>件
[会計先名] <% shp_name %>
[通信欄]
<% bil_req1 %>

[支払通信欄]
<% bil_note %>

ご連絡先
------------------------------------------------------------------------
[お名前] <% adr_name1 %> <% adr_name2 %> (<% adr_kana1 %> <% adr_kana2 %>) 様
[郵便番号] <% adr_zip.zipcode %>
[住所]     <% adr_are_name %><% adr_city %>
           <% adr_town %><% adr_etc %>
[電話番号] <% adr_tel %>
[FAX番号]  <% adr_fax %>
[メールアドレス] <% bil_email %>

<% IF slt_note2 -%>
<% pay_name %>/<% dvy_name %>について
------------------------------------------------------------------------
<% slt_note2 %>
<% ELSE -%>
<% IF slt_note1 -%>
<% pay_name %>/<% dvy_name %>について
------------------------------------------------------------------------
<% slt_note1 %>
<% END -%>
<% END -%>

ご注文商品
------------------------------------------------------------------------
<% FOR OrderItems -%>
[商品登録コード: <% item_id %>]
<% odr_gds_name %> <% odr_gds_options %>
@<% odr_gds_bprice %> x <% odr_number %> = <% odr_bprice %>(円)
<% END -%>
------------------------------------------------------------------------

<% FOR OrderSell -%>
お届け先(<% :count %>)
------------------------------------------------------------------------
<% IF sel_ticket -%>
[お名前] <% sel_to %> 様
[メールアドレス] <% sel_email %>
[タイトル] <% sel_subject %>
[メッセージ]
<% sel_message %>
[商品の受取り] <% IF sel_atime %><% IF sel_auth %>承諾<% ELSE %>拒否<% END %>(<% sel_atime %>)<% ELSE %>未確認<% END %>
<% ELSE -%>
[お名前] <% adr_name1 %> <% adr_name2 %> (<% adr_kana1 %> <% adr_kana2 %>) 様
[郵便番号] <% adr_zip.zipcode %>
[住所]     <% adr_are_name %><% adr_city %>
           <% adr_town %><% adr_etc %>
[電話番号] <% adr_tel %>
[FAX番号]  <% adr_fax %>
[時間帯指定] <% sel_req1 %>
<% END -%>
<% IF $slt_gift -%>
[ラッピングサービス] <% IF sel_gift %>利用する<% ELSE %>利用しない<% END %>
<% IF sel_gift %>[ラッピングメッセージ]
<% sel_req2 %>
<% END -%>
<% END -%>
[出荷通信欄]
<% sel_note %>
------------------------------------------------------------------------
商品小計                <% sel_bprice %>(円)
地域別送料              <% sel_pcharge %>(円)
離島追加送料            <% sel_icharge %>(円)
商品追加送料            <% sel_acharge %>(円)
クール便料              <% sel_ccharge %>(円)
ラッピング料            <% sel_gcharge %>(円)
小計                    <% sel_price %>(円)
------------------------------------------------------------------------
<% END -%>

お支払い
------------------------------------------------------------------------
商品合計                <% bil_bprice %>(円) (内税 <% bil_tprice %>円)
調整金額                <% bil_fprice %>(円)
地域別送料合計          <% bil_pcharge %>(円)
離島追加送料合計        <% bil_icharge %>(円)
商品追加送料合計        <% bil_acharge %>(円)
クール便料合計          <% bil_ccharge %>(円)
ラッピング料合計        <% bil_gcharge %>(円)
決済手数料              <% bil_fcharge %>(円)
ポイント割引            <% IF bil_pprice %>-<% bil_pprice %><% ELSE %>0<% END %>(円)
------------------------------------------------------------------------
お支払い合計            <% bil_price %>(円)
------------------------------------------------------------------------

ご注文内容・注文履歴は以下のホームページでもご確認頂けます。
<% cgi_customer %>/0/history.html

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
------------------------------------------------------------------------
TIME            : <% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用注文受付メール
 {
  name => 'confirm_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'confirm',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 2,
			cml_open => 1,
			cml_name => '注文受付',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>ご注文の受付確認(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

ご注文頂きありがとうございます。
ご注文を受付けいたしました。お支払い内容をご確認下さい。

●ご注文内容
[注文日]
<% bil_create.timef(%04d/%02d/%02d %02d:%02d) %>
[注文番号]
<% order_id %>
[支払方法]
<% bil_pay_name %>
[配送方法]
<% bil_dvy_name %>
[調整金額]
<% bil_fprice %>(円)
[お支払い合計]
<% bil_price %>(円)
[通信欄]
<% bil_req1 %>
[支払通信欄]
<% bil_note %>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.xhtml

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用支払完了メール
 {
  name => 'payment_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'payment',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 3,
			cml_open => 1,
			cml_name => '支払完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>お支払いを確認しました。(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

ご注文頂きありがとうございます。
お支払いを確認いたしました。

●お支払い内容
[注文番号] <% order_id %>
<% IF bil_uid %>[支払伝票番号] <% bil_uid %>
<% END -%>
[支払方法] <% bil_pay_name %>
[お支払い金額] <% bil_price %>(円)

<% IF bil_note %><% bil_note %>
<% END -%>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.html

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用支払完了メール
 {
  name => 'payment_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'payment',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 3,
			cml_open => 1,
			cml_name => '支払完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>お支払いを確認しました。(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

ご注文頂きありがとうございます。
お支払いを確認いたしました。

●お支払い内容
[注文番号]
<% order_id %>
<% IF bil_uid %>[支払伝票番号]
<% bil_uid %>
<% END -%>
[支払方法]
<% bil_pay_name %>
[お支払い金額]
<% bil_price %>(円)

<% IF bil_note %><% bil_note %>
<% END -%>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.xhtml

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用配送予定メール
 {
  name => 'appoint_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'appoint',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 4,
			cml_open => 1,
			cml_name => '配送予定',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>配送予定のお知らせ(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% bil_adr_name1 %> <% bil_adr_name2 %> 様

ご注文頂きありがとうございます。
発送完了までしばらくお待ち下さい。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号] <% order_id %>
[配送方法] <% bil_dvy_name %>
[配送先] <% IF sel_ticket %><% sel_to %><% ELSE %><% adr_name1 %> <% adr_name2 %><% END %> 様
[出荷予定日] <% sel_appoint %>

●ご注文商品
<% FOR OrderItems -%>
[商品登録コード: <% item_id %>]
<% odr_gds_name %> <% odr_gds_options %>
@<% odr_gds_bprice %> x <% odr_number %> = <% odr_bprice %>(円)
<% END -%>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.html

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用配送予定メール
 {
  name => 'appoint_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'appoint',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 4,
			cml_open => 1,
			cml_name => '配送予定',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>配送予定のお知らせ(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% bil_adr_name1 %> <% bil_adr_name2 %> 様

ご注文頂きありがとうございます。
発送完了までしばらくお待ち下さい。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号]
<% order_id %>
[配送方法]
<% bil_dvy_name %>
[配送先]
<% IF sel_ticket %><% sel_to %><% ELSE %><% adr_name1 %> <% adr_name2 %><% END %> 様
[出荷予定日]
<% sel_appoint %>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.xhtml

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用配送完了メール
 {
  name => 'delivery_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'delivery',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 5,
			cml_open => 1,
			cml_name => '配送完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>配送完了のお知らせ(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% bil_adr_name1 %> <% bil_adr_name2 %> 様

ご注文頂きありがとうございます。
商品の発送が完了しました。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号] <% order_id %>
[配送方法] <% bil_dvy_name %>
[配送先] <% IF sel_ticket %><% sel_to %><% ELSE %><% adr_name1 %> <% adr_name2 %><% END %> 様
<% IF sel_uid %>[配送伝票番号] <% sel_uid %>
<% END -%>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.html

お届けしました商品について「商品レビュー」のご投稿をお待ちしております。
ご投稿は各商品詳細ページの「この商品のレビューを投稿する」から行うこと
ができます。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用配送完了メール
 {
  name => 'delivery_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'delivery',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 5,
			cml_open => 1,
			cml_name => '配送完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>配送完了のお知らせ(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% bil_adr_name1 %> <% bil_adr_name2 %> 様

ご注文頂きありがとうございます。
商品の発送が完了しました。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号]
<% order_id %>
[配送方法]
<% bil_dvy_name %>
[配送先]
<% IF sel_ticket %><% sel_to %><% ELSE %><% adr_name1 %> <% adr_name2 %><% END %> 様
<% IF sel_uid %>[配送伝票番号]
<% sel_uid %>
<% END -%>

ご注文内容・注文履歴はホームページでご確認頂けます。
<% cgi_customer %>/0/history.xhtml

お届けしました商品について「商品レビュー」のご投稿をお待ちしております。
ご投稿は各商品詳細ページの「この商品のレビューを投稿する」から行うこと
ができます。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF


									},
						  },
		   },
 },
 # PC用メールギフト通知
 {
  name => 'ticket_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'ticket',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 6,
			cml_open => 1,
			cml_name => 'メールギフト通知',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% sel_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_reply => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト/<% sel_to %>様(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_to %> 様
<% sel_from %> 様よりメールギフトをお預かりしています。

[メッセージ] <% sel_subject %>
<% sel_message %>

[送り主] <% sel_from %>

[ギフト商品]
------------------------------------------------------------------------
<% FOR OrderItems -%>
[商品登録コード: <% item_id %>] <% gds_name %> <% gds_options %>
<% END -%>
------------------------------------------------------------------------
メールギフトは送り主にあなたの住所を知らせずにギフト商品を受け取ることが
できるサービスです。ギフト商品の代金及び送料・手数料は送り主が負担します。

以下のURLをご欄頂き、次のいずれかのお手続きをお願いいたします。
<% cgi_order %>/0/ticket.html?id=<% sel_hash %>&cd=<% passwd %>&login=1

●ギフト商品の受取りを承諾される場合
商品のお届け先を入力して下さい。

●ギフト商品の受取りを拒否される場合
お手数ですが受取り拒否手続きをお願いいたします。

なお上記のお手続きがないまま7日以上経過した場合は、無効(受取り拒否扱い)と
させて頂きますので、予めご了承下さい。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
------------------------------------------------------------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用メールギフト通知
 {
  name => 'ticket_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'ticket',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 6,
			cml_open => 1,
			cml_name => 'メールギフト通知',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% sel_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_reply => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト/<% sel_to %>様(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_to %> 様
<% sel_from %> 様よりメールギフトをお預かりしています。

[メッセージ] <% sel_subject %>
<% sel_message %>

[送り主]
<% sel_from %>

[ギフト商品]
<% FOR OrderItems -%>
[商品登録コード: <% item_id %>] <% gds_name %> <% gds_options %>
<% END -%>

メールギフトは送り主にあなたの住所を知らせずにギフト商品を受け取ることができるサービスです。
ギフト商品の代金及び送料・手数料は送り主が負担します。

以下のURLをご欄頂き、次のいずれかのお手続きをお願いいたします。
<% cgi_order %>/0/ticket.xhtml?id=<% sel_hash %>&cd=<% passwd %>&login=1

●ギフト商品の受取りを承諾される場合
商品のお届け先を入力して下さい。

●ギフト商品の受取りを拒否される場合
お手数ですが受取り拒否手続きをお願いいたします。

なお上記のお手続きがないまま7日以上経過した場合は、無効(受取り拒否扱い)とさせて頂きますので、予めご了承下さい。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用メールギフト確認
 {
  name => 'confirm_ticket_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'confirm_ticket',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 7,
			cml_open => 1,
			cml_name => 'メールギフト確認',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_reply => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト確認/自動配信メール(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_from %> 様

ご利用頂きありがとうございます。
<% IF sel_auth -%>
<% sel_to %> 様がギフト商品の受取りを承諾されました。
送料・手数料込みのお支払い金額は、後ほどメールでお知らせいたします。
お支払いは合計金額が確定した後にお願いいたします。
<% ELSE -%>
<% sel_to %> 様がギフト商品の受取りを拒否されました。
<% END -%>

[返信メッセージ]
<% sel_answer %>

[ギフト商品]
------------------------------------------------------------------------
<% FOR OrderItems -%>
[商品登録コード: <% item_id %>] <% odr_gds_name %> <% odr_gds_options %>
<% END -%>
------------------------------------------------------------------------

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
------------------------------------------------------------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用メールギフト確認
 {
  name => 'confirm_ticket_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'confirm_ticket',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 7,
			cml_open => 1,
			cml_name => 'メールギフト確認',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_reply => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト確認/自動配信メール(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_from %> 様

ご利用頂きありがとうございます。
<% IF sel_auth -%>
<% sel_to %> 様がギフト商品の受取りを承諾されました。
送料・手数料込みのお支払い金額は、後ほどメールでお知らせいたします。
お支払いは合計金額が確定した後にお願いいたします。
<% ELSE -%>
<% sel_to %> 様がギフト商品の受取りを拒否されました。
<% END -%>

[返信メッセージ]
<% sel_answer %>

[ギフト商品]
<% FOR OrderItems -%>
[商品登録コード: <% item_id %>] <% odr_gds_name %> <% odr_gds_options %>
<% END -%>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用メールギフト配送予定メール
 {
  name => 'appoint_ticket_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'appoint_ticket',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 8,
			cml_open => 1,
			cml_name => 'メールギフト配送予定',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% sel_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト配送予定のお知らせ/<% sel_to %>様(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_to %> 様

メールギフトサービスをご利用頂きありがとうございます。
発送完了までしばらくお待ち下さい。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号] <% order_id %>
[出荷予定日] <% sel_appoint %>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用メールギフト配送予定メール
 {
  name => 'appoint_ticket_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'appoint_ticket',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 8,
			cml_open => 1,
			cml_name => 'メールギフト配送予定',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% sel_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト配送予定のお知らせ/<% sel_to %>様(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_to %> 様

メールギフトサービスをご利用頂きありがとうございます。
発送完了までしばらくお待ち下さい。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号]
<% order_id %>
[出荷予定日]
<% sel_appoint %>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用メールギフト配送完了メール
 {
  name => 'delivery_ticket_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'delivery_ticket',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 9,
			cml_open => 1,
			cml_name => 'メールギフト配送完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% sel_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト配送完了のお知らせ/<% sel_to %>様(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_to %> 様

メールギフトサービスをご利用頂きありがとうございます。
商品の発送が完了しました。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号] <% order_id %>
<% IF sel_uid %>[配送伝票番号] <% sel_uid %>
<% END -%>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用メールギフト配送完了メール
 {
  name => 'delivery_ticket_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'delivery_ticket',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 9,
			cml_open => 1,
			cml_name => 'メールギフト配送完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% sel_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>メールギフト配送完了のお知らせ/<% sel_to %>様(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% sel_to %> 様

メールギフトサービスをご利用頂きありがとうございます。
商品の発送が完了しました。
<% IF sel_note %><% sel_note %>
<% END -%>

●ご注文内容
[注文番号]
<% order_id %>
<% IF sel_uid %>[配送伝票番号]
<% sel_uid %>
<% END -%>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF

									},
						  },
		   },
 },
 # PC用会員登録/更新メール
 {
  name => 'customer_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'customer',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 10,
			cml_open => 0,
			cml_name => '会員の登録/更新',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>会員<% IF updated %>更新<% ELSE %><% IF entry %>仮<% END %>登録<% END %>完了',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
ご<% IF updated %>利用<% ELSE %>登録<% END %>ありがとうございます。

お名前              : <% adr_name1 %> <% adr_name2 %>(<% adr_kana1 %><% adr_kana2 %>)
都道府県            : <% adr_are_name %>
メールアドレス      : <% ctm_email %>

<% IF entry -%>
今回お申し込み頂いた内容は仮登録の状態です。
以下のページにアクセスして登録を完了して下さい。
<% cgi_customer %>/0/confirm.html?id=<% verify_id %>&next=0/finish
※このURLは数時間経過すると無効になります。
※アクセスされる場合はお申込み時のブラウザをご使用下さい。
※登録した覚えのない方は無視して下さい。
<% ELSE -%>
ご登録内容の詳細は以下のマイページでご確認下さい。
<% cgi_customer %>/0/menu.html
<% END -%>
<% IF updated %>
パスワードを変更される場合は、以下の URL にアクセスして設定して下さい。
<% cgi_customer %>/0/passwd.html
<% END -%>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
<% shp_note1 %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # 携帯用会員登録/更新メール
 {
  name => 'customer_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'customer',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 10,
			cml_open => 0,
			cml_name => '会員の登録/更新',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>会員<% IF updated %>更新<% ELSE %><% IF entry %>仮<% END %>登録<% END %>完了',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
ご<% IF updated %>利用<% ELSE %>登録<% END %>ありがとうございます。

お名前: <% adr_name1 %> <% adr_name2 %>(<% adr_kana1 %><% adr_kana2 %>)
都道府県: <% adr_are_name %>

<% IF entry -%>
今回お申し込み頂いた内容は仮登録の状態です。
以下のページにアクセスして登録を完了して下さい。
<% cgi_customer %>/0/confirm.xhtml?rid=<% rid %>&id=<% verify_id %>&next=0/finish
※このURLは数時間経過すると無効になります。
※アクセスされる場合はお申込み時のブラウザをご使用下さい。
※登録した覚えのない方は無視して下さい。
<% ELSE -%>
ご登録内容の詳細は以下のマイページでご確認下さい。
<% cgi_customer %>/0/menu.xhtml
<% END -%>
<% IF updated %>
パスワードを変更される場合は、以下の URL にアクセスして設定して下さい。
<% cgi_customer %>/0/passwd.xhtml
<% END -%>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
<% shp_note1 %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # PC用パスワード設定メール
 {
  name => 'verify_customer_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'verify_customer',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 11,
			cml_open => 0,
			cml_name => '会員パスワードの設定',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>パスワードの設定',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様
ご利用頂きありがとうございます。

パスワードは、以下の URL にアクセスして設定して下さい。
<% cgi_customer %>/0/verify.html?id=<% verify_id %>

※このURLは数時間経過すると無効になります。
※アクセスされる場合はお申込み時のブラウザをご使用下さい。

■「有効期限切れ」が表示される場合の対処方法
※ブラウザのURL欄に上記のURLが表示されているかご確認下さい。
  リンクの途中まで表示されている場合は、再度上記URLを入力してページをご欄下さい。
※お申し込み時のブラウザを使用されているかご確認下さい。
  他のブラウザを使用してアクセスされた場合は、リンクが無効になりますので
  再度お申し込み下さい。
※お申し込みから数時間経過していないかご確認下さい。
  URLは一定時間経過すると無効になりますので、再度お申し込み下さい。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # 携帯用パスワード設定メール
 {
  name => 'verify_customer_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'verify_customer',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 11,
			cml_open => 0,
			cml_name => '会員パスワードの設定',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>パスワードの設定',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様
ご利用頂きありがとうございます。

パスワードは、以下の URL にアクセスして設定して下さい。
<% cgi_customer %>/0/verify.xhtml?id=<% verify_id %>&rid=<% rid %>

※このURLは数時間経過すると無効になります。
※アクセスされる場合はお申込み時のブラウザをご使用下さい。

■「有効期限切れ」が表示される場合の対処方法
※ブラウザのURL欄に上記のURLが表示されているかご確認下さい。
  リンクの途中まで表示されている場合は、再度上記URLを入力してページをご欄下さい。
※お申し込み時のブラウザを使用されているかご確認下さい。
  他のブラウザを使用してアクセスされた場合は、リンクが無効になりますので
  再度お申し込み下さい。
※お申し込みから数時間経過していないかご確認下さい。
  URLは一定時間経過すると無効になりますので、再度お申し込み下さい。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # PC用退会通知メール
 {
  name => 'refuse_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'refuse',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 12,
			cml_open => 0,
			cml_name => '会員の退会',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>退会完了のお知らせ',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

顧客番号      : <% ctm_id %>
メールアドレス: <% ctm_email %>

退会処理が完了しました。

ご利用ありがとうございました。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # 携帯用退会通知メール
 {
  name => 'refuse_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'refuse',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 12,
			cml_open => 0,
			cml_name => '会員の退会',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>退会完了のお知らせ',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

顧客番号      : <% ctm_id %>
メールアドレス: <% ctm_email %>

退会処理が完了しました。

ご利用ありがとうございました。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # PC用ポイント発行通知メール
 {
  name => 'point_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'point',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 13,
			cml_open => 0,
			cml_name => 'ポイント発行のお知らせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>ポイント発行のお知らせ(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様
ご利用頂きありがとうございます。
ご注文分のポイントが発行されました。

ご注文番号        : <% order_id %>
今回の発行ポイント: <% ctm_point2 %>
現在の所有ポイント: <% ctm_point %>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用ポイント発行通知メール
 {
  name => 'point_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'point',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 13,
			cml_open => 0,
			cml_name => 'ポイント発行のお知らせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% bil_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>ポイント発行のお知らせ(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様
ご利用頂きありがとうございます。
ご注文分のポイントが発行されました。

ご注文番号        : <% order_id %>
今回の発行ポイント: <% ctm_point2 %>
現在の所有ポイント: <% ctm_point %>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用お問い合わせメール
 {
  name => 'contact_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'contact',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 15,
			cml_open => 1,
			cml_name => 'お問い合わせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% shp_email %>',
									 eml_cc => '<% req_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% req_email %>',
									 eml_subject => '<<% shp_name %>><% req_subject %>',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
お問い合わせ頂きありがとうございます。

お名前         : <% req_name %>
メールアドレス : <% req_email %>
お問合わせ内容 :

<% req_message %>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # 携帯用お問い合わせメール
 {
  name => 'contact_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'contact',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 15,
			cml_open => 1,
			cml_name => 'お問い合わせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% shp_email %>',
									 eml_cc => '<% req_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% req_email %>',
									 eml_subject => '<<% shp_name %>><% req_subject %>',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
お問い合わせ頂きありがとうございます。

お名前
<% req_name %>

メールアドレス
<% req_email %>

お問合わせ内容
<% req_message %>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用友達に知らせるメール
 {
  name => 'notice_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'notice',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 16,
			cml_open => 0,
			cml_name => '友達に知らせる',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% fyi_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% ctm_email %>',
									 eml_subject => '<<% shp_name %>>お知らせ/<% fyi_handle %>様より',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% fyi_handle %>さんからのお知らせメールです。

<% fyi_message -%>

<% IF gds_id -%>
<% gds_name %>
<% cgi_search %>/<% item_gmd %>/item.html
<% END -%>

<% shp_name %>
<% shp_url %>/
メールアドレス:<% shp_email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
EOF
									},
						  },
		   },
 },
 # 携帯用友達に知らせるメール
 {
  name => 'notice_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'notice',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 16,
			cml_open => 0,
			cml_name => '友達に知らせる',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% fyi_email %>',
									 eml_cc => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% ctm_email %>',
									 eml_subject => '<<% shp_name %>>お知らせ/<% fyi_handle %>様より',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% fyi_handle %>さんからのお知らせメールです。

<% fyi_message -%>

<% IF gds_id -%>
<% gds_name %>
<% cgi_search %>/<% item_gmd %>/item.xhtml
<% END -%>

<% shp_name %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用レビュー投稿のお知らせメール
 {
  name => 'review_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'review',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 18,
			cml_open => 0,
			cml_name => 'レビュー投稿',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% shp_email %>',
									 eml_subject => '<<% shp_name %>>レビュー投稿のお知らせ',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
「<% IF gds_id %><% gds_name %><% ELSE %><% shp_name %><% END %>」についてのレビュー記事が投稿されました。

投稿者名   : <% rev_handle %>
年齢       : <% IF rev_age %><% rev_age %>0年代<% END %>
性別       : <% rev_sex.sex_name %>
おすすめ度 : <% rev_point.times(★) %>
タイトル   : <% rev_subject %>

<% rev_summary %>
<% rev_message %>

※投稿されたレビュー記事を公開するには管理室で設定して下さい。
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # ショップのパスワード設定メール(PCのみ)
 {
  name => 'verify_shop_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'verify_shop',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 20,
			cml_open => 0,
			cml_name => 'ショップパスワードの設定',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>パスワードの設定',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
ご利用頂きありがとうございます。

パスワードは、以下の URL にアクセスして設定して下さい。
<% FOR SHPList %>
<% shp_name %>
<% $cgi_contact %>/0/verify.html?id=<% $verify_id %>&shp_id=<% shp_id %>
<% END %>
※このURLは数時間経過すると無効になります。
※アクセスされる場合はお申込み時のブラウザをご使用下さい。

■「有効期限切れ」が表示される場合の対処方法
※ブラウザのURL欄に上記のURLが表示されているかご確認下さい。
  リンクの途中まで表示されている場合は、再度上記URLを入力してページをご欄下さい。
※お申し込み時のブラウザを使用されているかご確認下さい。
  他のブラウザを使用してアクセスされた場合は、リンクが無効になりますので
  再度お申し込み下さい。
※お申し込みから数時間経過していないかご確認下さい。
  URLは一定時間経過すると無効になりますので、再度お申し込み下さい。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # 在庫不足メール
 {
  name => 'stock_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'stock',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 30,
			cml_open => 0,
			cml_name => '商品在庫切れのお知らせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>商品在庫切れのお知らせ',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% shp_name %> <% shp_admin %> 様

モール管理システムよりお知らせです。

以下の商品の僅少、もしくは売り切れが発生しています。在庫が 0 になりますと
お客様にはご購入頂けない商品として取り扱われます。在庫の補充もしくは、掲
載されている商品の削除をお勧め致します。

<% FOR StockItems -%>
[商品番号: <% gds_id %>]
<% gds_name %> <% gds_option1 %><% gds_option2 %><% gds_option3 %><% gds_option4 %>
在庫数 <% gds_stock %>
<% END -%>

-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # PC用入荷通知メール
 {
  name => 'arrival_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'arrival',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 35,
			cml_open => 0,
			cml_name => '商品入荷のお知らせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>商品入荷のお知らせ',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

以下の商品が入荷しましたのでお知らせいたします。
<% FOR Items %>
[商品登録コード: <% item_id %>] <% gds_name %> <% gds_options %>
<% $cgi_search %>/<% item_gmd %>/item.html
<% END -%>

このメールは、現在買い物かごに入っている商品を対象に
<% datetime.timef(%04d/%02d/%02d %02d:%02d) %> 時点の在庫状況にもとづいてお知らせしています。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% shp_url %>/
メールアドレス:<% shp_email %>
EOF
									},
						  },
		   },
 },
 # 携帯用入荷通知メール
 {
  name => 'arrival_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'arrival',
			cml_shp_id => 0,
			cml_trm_id => 2,
		   ],
  param => {
			cml_rank => 35,
			cml_open => 0,
			cml_name => '商品入荷のお知らせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>商品入荷のお知らせ',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% adr_name1 %> <% adr_name2 %> 様

以下の商品が入荷しましたのでお知らせいたします。
<% FOR Items %>
[商品登録コード: <% item_id %>] <% gds_name %><% gds_options %>
<% $cgi_search %>/<% item_gmd %>/item.xhtml
<% END -%>

このメールは、現在買い物かごに入っている商品を対象に
<% datetime.timef(%04d/%02d/%02d %02d:%02d) %> 時点の在庫状況にもとづいてお知らせしています。

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
EOF
									},
						  },
		   },
 },
 # 決済サービスの支払完了メール
 {
  name => 'callback_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'callback',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 40,
			cml_open => 0,
			cml_name => '決済サービスの支払完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>お支払完了/自動配信メール(<% order_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% shp_name %> <% shp_admin %> 様

モール管理システムよりお知らせです。

[注文番号] <% order_id %>
[支払方法] <% bil_pay_name %>
[決済サービス名] <% psv_name %>

<% bil_price %>(円)

上記のお支払いが完了しました。

決済結果
<% psv_totmpl %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # 携帯サイトURL通知メール
 {
  name => 'easy_xhtml',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'easy',
			cml_shp_id => 0,
			cml_trm_id => 2
		   ],
  param => {
			cml_rank => 50,
			cml_open => 0,
			cml_name => '携帯サイトURLのお知らせ',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% ctm_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>携帯サイト<% IF ctm_id %>(簡単アクセス)<% END %>',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
ご利用ありがとうございます。
以下の URL にアクセスして画面の登録を行って下さい。

<% IF ctm_id -%>
<% cgi_static %>/0/login.xhtml?e=<% ctm_email.escape %>&i=<% hash %>&_next=0/index
<% ELSE -%>
<% cgi_static %>/0/index.xhtml
<% END -%>

<% shp_name %>
〒<% shp_zip.zipcode %>
<% shp_are_name %><% shp_city %><% shp_town %><% shp_etc %>
Tel:<% shp_tel %>/Fax:<% shp_fax %>
<% cgi_static %>/<% shp_id %>/index.xhtml
メールアドレス:<% shp_email %>
----------------------------------------------------------------------
TIME            : <% datetime %>
REMOTE_HOST     : <% REMOTE_HOST %>
REMOTE_ADDR     : <% REMOTE_ADDR %>
HTTP_REFERER    : <% HTTP_REFERER %>
HTTP_USER_AGENT : <% HTTP_USER_AGENT %>
EOF
									},
						  },
		   },
 },
 # ショップ利用料の請求/支払完了メール
 {
  name => 'charge_html',
  class => 'ESP::SMS::Data::CML',
  where => [
			cml_type => 'charge',
			cml_shp_id => 0,
			cml_trm_id => 1
		   ],
  param => {
			cml_rank => 60,
			cml_open => 0,
			cml_name => '利用料の請求/支払完了',
			cml_eml_id => {
						   class => 'ESP::SMS::Data::EML',
						   join => 'eml_id',
						   param => {
									 eml_shp_id => 0,
									 eml_to => '<% shp_email %>',
									 eml_bcc => '<% email %>',
									 eml_from => '<% email %>',
									 eml_subject => '<<% shp_name %>>利用料の<% IF chr_status %>お支払完了<% ELSE %>ご請求<% END %>(<% chr_year %>年<% chr_month %>月分/<% charge_id %>)',
									 eml_body => <<'EOF',
<% LOAD .TIME.URL.ENV -%>
<% shp_name %> <% shp_admin %> 様

ご利用頂きありがとうございます。
<% IF chr_status -%>
<% chr_year %>年<% chr_month %>月分のお支払いを確認いたしました。
<% ELSE -%>
<% chr_year %>年<% chr_month %>月分のご請求をお知らせいたします。
<% END -%>

●お支払い内容
[請求番号] <% charge_id %>
<% IF chr_uid %>[支払伝票番号] <% chr_uid %>
<% END -%>
[お支払い金額] <% chr_price %>(円)

<% IF chr_note %><% chr_note %>
<% END -%>
<% IF chr_open %>詳細は以下のページでご確認下さい。
<% cgi_chrmgr %>/<% chr_shp_id %>/index.html
<% END -%>

-------------------
<% datetime %>
EOF
									},
						  },
		   },
 },
 # 規約
 {
  name => 'law1',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 1
		   ],
  param => {
			law_title => '業務内容',
			law_message => '',
		   },
 },
 {
  name => 'law2',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 2
		   ],
  param => {
			law_title => '商品代金以外の必要金額',
			law_message => '支払方法、配送方法、配送地域に応じて別途料金が必要です。',
		   },
 },
 {
  name => 'law3',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 3
		   ],
  param => {
			law_title => '申し込みの有効期限',
			law_message => '',
		   },
 },
 {
  name => 'law4',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 4
		   ],
  param => {
			law_title => '不良品',
			law_message => '',
		   },
 },
 {
  name => 'law5',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 5
		   ],
  param => {
			law_title => '引渡し期限・納期',
			law_message => '',
		   },
 },
 {
  name => 'law6',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 6
		   ],
  param => {
			law_title => '返品期限',
			law_message => '',
		   },
 },
 {
  name => 'law7',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 7
		   ],
  param => {
			law_title => '返品・交換送料',
			law_message => '',
		   },
 },
 {
  name => 'law8',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 8
		   ],
  param => {
			law_title => '',
			law_message => '',
			law_open => 0,
		   },
 },
 {
  name => 'law9',
  class => 'ESP::SMS::Data::LAW',
  where => [
			law_shp_id => 0,
			law_oid => 9
		   ],
  param => {
			law_title => '',
			law_message => '',
			law_open => 0,
		   },
 },
 # 初期ジャンル
 {
  name => 'gnr1',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 1
		   ],
  param => {
			gnr_name => '注目商品',
		   },
 },
 {
  name => 'gnr2',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 2
		   ],
  param => {
			gnr_name => 'パソコン・周辺機器',
		   },
 },
 {
  name => 'gnr3',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 3
		   ],
  param => {
			gnr_name => '家電・AV',
		   },
 },
 {
  name => 'gnr4',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 4
		   ],
  param => {
			gnr_name => 'CD・DVD',
		   },
 },
 {
  name => 'gnr5',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 5
		   ],
  param => {
			gnr_name => '書籍・雑誌',
		   },
 },
 {
  name => 'gnr6',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 6
		   ],
  param => {
			gnr_name => 'ホビー・ゲーム',
		   },
 },
 {
  name => 'gnr7',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 7
		   ],
  param => {
			gnr_name => 'インテリア・生活用品',
		   },
 },
 {
  name => 'gnr8',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 8
		   ],
  param => {
			gnr_name => '食品・飲料',
		   },
 },
 {
  name => 'gnr9',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 9
		   ],
  param => {
			gnr_name => 'ファッション・衣料',
		   },
 },
 {
  name => 'gnr10',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 10
		   ],
  param => {
			gnr_name => '健康・美容・コスメ',
		   },
 },
 {
  name => 'gnr11',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 11
		   ],
  param => {
			gnr_name => 'スポーツ・アウトドア',
		   },
 },
 {
  name => 'gnr12',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 12
		   ],
  param => {
			gnr_name => '車・バイク',
		   },
 },
 {
  name => 'gnr13',
  class => 'ESP::SMS::Data::GNR',
  where => [
			gnr_id => 13
		   ],
  param => {
			gnr_name => 'ペット',
		   },
 },
 # 初期カテゴリ
 {
  name => 'grp1',
  class => 'ESP::SMS::Data::GRP',
  where => [
			grp_id => 1
		   ],
  param => {
			grp_shp_id => 0,
			grp_gnr_id => 1,
			grp_name => 'おすすめ商品',
		   },
 },
 {
  name => 'grp2',
  class => 'ESP::SMS::Data::GRP',
  where => [
			grp_id => 2
		   ],
  param => {
			grp_shp_id => 0,
			grp_gnr_id => 1,
			grp_name => 'お買い得商品',
		   },
 },
 # 決済方法(PC/銀行振込/宅配便)
 {
  name => 'slt1',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 1,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '銀行振込の場合は、ご注文後○日以内にお支払い下さい。
【振込先】 ◯◯銀行 ◯◯支店 普通 1234567 ◯◯◯◯
※入金確認後の商品手配となります。
※振込支払い時の手数料はお客様負担となります。
※◯日以内にご入金がない場合、注文はキャンセルとさせていただきます。
',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/銀行振込/宅配便)-条件1
 {
  name => 'fee1_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 1,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(PC/郵便振替/宅配便)
 {
  name => 'slt2',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 2,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '郵便振替の場合は、ご注文後○日以内にお支払い下さい。
【振込先】 12345-00011111 ◯◯◯◯　◯◯◯
※入金確認後の商品手配となります。
※振込支払い時の手数料はお客様負担となります。
※◯日以内にご入金がない場合、注文はキャンセルとさせていただきます。
',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/郵便振替/宅配便)-条件1
 {
  name => 'fee2_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 2,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(PC/代金引替/宅配便)
 {
  name => 'slt3',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 3,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 300000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '代金引替の場合は、商品到着時に運送会社の方に代金をお支払い下さい。',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/代金引替/宅配便)-条件1
 {
  name => 'fee3_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 315,
			fee_unit => undef,
		   },
 },
 # 手数料(PC/代金引替/宅配便)-条件2
 {
  name => 'fee3_2',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 2,
		   ],
  param => {
			fee_price => 10000,
			fee_charge => 420,
			fee_unit => undef,
		   },
 },
 # 手数料(PC/代金引替/宅配便)-条件3
 {
  name => 'fee3_3',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 3,
		   ],
  param => {
			fee_price => 30000,
			fee_charge => 630,
			fee_unit => undef,
		   },
 },
 # 手数料(PC/代金引替/宅配便)-条件4
 {
  name => 'fee3_4',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 4,
		   ],
  param => {
			fee_price => 100000,
			fee_charge => 1050,
			fee_unit => undef,
		   },
 },
 # 決済方法(PC/コンビニ決済/宅配便)
 {
  name => 'slt4',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 4,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 300000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/コンビニ決済/宅配便)-条件1
 {
  name => 'fee4_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 150,
			fee_unit => undef,
		   },
 },
 # 手数料(PC/コンビニ決済/宅配便)-条件2
 {
  name => 'fee4_2',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 2,
		   ],
  param => {
			fee_price => 2000,
			fee_charge => 170,
			fee_unit => undef,
		   },
 },
 # 手数料(PC/コンビニ決済/宅配便)-条件3
 {
  name => 'fee4_3',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 3,
		   ],
  param => {
			fee_price => 3000,
			fee_charge => 200,
			fee_unit => undef,
		   },
 },
 # 手数料(PC/コンビニ決済/宅配便)-条件4
 {
  name => 'fee4_4',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 4,
		   ],
  param => {
			fee_price => 10000,
			fee_charge => 250,
			fee_unit => undef,
		   },
 },
 # 手数料(PC/コンビニ決済/宅配便)-条件5
 {
  name => 'fee4_5',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 5,
		   ],
  param => {
			fee_price => 30000,
			fee_charge => 500,
			fee_unit => undef,
		   },
 },
 # 決済方法(PC/コンビニ決済/店頭受取り)
 {
  name => 'slt5',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 4,
			slt_dvy_id => 10
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 300000,
			slt_dvymin => 0,
			slt_dvymax => 0,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 0,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/コンビニ決済/店頭受取り)-条件1
 {
  name => 'fee5_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 4,
									 slt_dvy_id => 10
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(PC/クレジット決済/宅配便)
 {
  name => 'slt6',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 5,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 1,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/クレジット決済/宅配便)-条件1
 {
  name => 'fee6_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 5,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 210,
			fee_unit => undef,
		   },
 },
 # 決済方法(PC/現金/店頭受取り)
 {
  name => 'slt7',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 6,
			slt_dvy_id => 10
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 0,
			slt_dvymax => 0,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 0,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/現金/店頭受取り)-条件1
 {
  name => 'fee7_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 6,
									 slt_dvy_id => 10
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(PC/銀行振込/メールギフト)
 {
  name => 'slt8',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 1,
			slt_pay_id => 1,
			slt_dvy_id => 5
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 0,
			slt_auth => 0,
			slt_tax => 0,
			slt_note1 => 'メールギフトは、注文者がお届け先の住所が分からない相手にメールアドレスを
利用して商品を発送するサービスです。
ご注文商品の代金及び送料・手数料は、注文者(送り主)が負担します。
商品の発送は、お届け先の相手が受取りを承認して、注文者が入金を行った後に
行われます。お支払いは合計金額が確定した後にお願いいたします。

【振込先】 ◯◯銀行 ◯◯支店 普通 1234567 ◯◯◯◯
※振込支払い時の手数料はお客様負担となります。
',
			slt_note2 => '',
		   },
 },
 # 手数料(PC/銀行振込/メールギフト)-条件1
 {
  name => 'fee8_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 1,
									 slt_pay_id => 1,
									 slt_dvy_id => 5
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/銀行振込/宅配便)
 {
  name => 'slt51',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 1,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '銀行振込の場合は、ご注文後○日以内にお支払い下さい。
【振込先】 ◯◯銀行 ◯◯支店 普通 1234567 ◯◯◯◯
※入金確認後の商品手配となります。
※振込支払い時の手数料はお客様負担となります。
※◯日以内にご入金がない場合、注文はキャンセルとさせていただきます。
',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/銀行振込/宅配便)-条件1
 {
  name => 'fee51_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 1,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/郵便振替/宅配便)
 {
  name => 'slt52',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 2,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '郵便振替の場合は、ご注文後○日以内にお支払い下さい。
【振込先】 12345-00011111 ◯◯◯◯　◯◯◯
※入金確認後の商品手配となります。
※振込支払い時の手数料はお客様負担となります。
※◯日以内にご入金がない場合、注文はキャンセルとさせていただきます。
',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/郵便振替/宅配便)-条件1
 {
  name => 'fee52_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 2,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/代金引替/宅配便)
 {
  name => 'slt53',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 3,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 300000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '代金引替の場合は、商品到着時に運送会社の方に代金をお支払い下さい。',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/代金引替/宅配便)-条件1
 {
  name => 'fee53_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 315,
			fee_unit => undef,
		   },
 },
 # 手数料(携帯/代金引替/宅配便)-条件2
 {
  name => 'fee53_2',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 2,
		   ],
  param => {
			fee_price => 10000,
			fee_charge => 420,
			fee_unit => undef,
		   },
 },
 # 手数料(携帯/代金引替/宅配便)-条件3
 {
  name => 'fee53_3',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 3,
		   ],
  param => {
			fee_price => 30000,
			fee_charge => 630,
			fee_unit => undef,
		   },
 },
 # 手数料(携帯/代金引替/宅配便)-条件4
 {
  name => 'fee53_4',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 3,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 4,
		   ],
  param => {
			fee_price => 100000,
			fee_charge => 1050,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/コンビニ決済/宅配便)
 {
  name => 'slt54',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 4,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 300000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/コンビニ決済/宅配便)-条件1
 {
  name => 'fee54_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 150,
			fee_unit => undef,
		   },
 },
 # 手数料(携帯/コンビニ決済/宅配便)-条件2
 {
  name => 'fee54_2',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 2,
		   ],
  param => {
			fee_price => 2000,
			fee_charge => 170,
			fee_unit => undef,
		   },
 },
 # 手数料(携帯/コンビニ決済/宅配便)-条件3
 {
  name => 'fee54_3',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 3,
		   ],
  param => {
			fee_price => 3000,
			fee_charge => 200,
			fee_unit => undef,
		   },
 },
 # 手数料(携帯/コンビニ決済/宅配便)-条件4
 {
  name => 'fee54_4',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 4,
		   ],
  param => {
			fee_price => 10000,
			fee_charge => 250,
			fee_unit => undef,
		   },
 },
 # 手数料(携帯/コンビニ決済/宅配便)-条件5
 {
  name => 'fee54_5',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 4,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 5,
		   ],
  param => {
			fee_price => 30000,
			fee_charge => 500,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/コンビニ決済/店頭受取り)
 {
  name => 'slt55',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 4,
			slt_dvy_id => 10
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 300000,
			slt_dvymin => 0,
			slt_dvymax => 0,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/コンビニ決済/店頭受取り)-条件1
 {
  name => 'fee55_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 4,
									 slt_dvy_id => 10
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/クレジット決済/宅配便)
 {
  name => 'slt56',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 5,
			slt_dvy_id => 1
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 1,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/クレジット決済/宅配便)-条件1
 {
  name => 'fee56_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 5,
									 slt_dvy_id => 1
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 210,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/現金/店頭受取り)
 {
  name => 'slt57',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 6,
			slt_dvy_id => 10
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 0,
			slt_dvymax => 0,
			slt_island => 0,
			slt_gift => 1,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => '',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/現金/店頭受取り)-条件1
 {
  name => 'fee57_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 6,
									 slt_dvy_id => 10
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 決済方法(携帯/銀行振込/メールギフト)
 {
  name => 'slt58',
  class => 'ESP::SMS::Data::SLT',
  where => [
			slt_shp_id => 0,
			slt_trm_id => 2,
			slt_pay_id => 1,
			slt_dvy_id => 5
		   ],
  param => {
			slt_open => 1,
			slt_paymin => 1,
			slt_paymax => 1000000,
			slt_dvymin => 1,
			slt_dvymax => 1,
			slt_island => 0,
			slt_gift => 0,
			slt_auth => 0,
			slt_tax => 1,
			slt_note1 => 'メールギフトは、注文者がお届け先の住所が分からない相手にメールアドレスを
利用して商品を発送するサービスです。
ご注文商品の代金及び送料・手数料は、注文者(送り主)が負担します。
商品の発送は、お届け先の相手が受取りを承認して、注文者が入金を行った後に
行われます。お支払いは合計金額が確定した後にお願いいたします。

【振込先】 ◯◯銀行 ◯◯支店 普通 1234567 ◯◯◯◯
※振込支払い時の手数料はお客様負担となります。
',
			slt_note2 => '',
		   },
 },
 # 手数料(携帯/銀行振込/メールギフト)-条件1
 {
  name => 'fee58_1',
  class => 'ESP::SMS::Data::FEE',
  where => [
			fee_shp_id => 0,
			fee_slt_id => {
						   class => 'ESP::SMS::Data::SLT',
						   join => 'slt_id',
						   where => [
									 slt_shp_id => 0,
									 slt_trm_id => 2,
									 slt_pay_id => 1,
									 slt_dvy_id => 5
									],
						  },
			fee_oid => 1,
		   ],
  param => {
			fee_price => 1,
			fee_charge => 0,
			fee_unit => undef,
		   },
 },
 # 送料
 # 宅配便/北海道(1)
 {
  name => 'pst1',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 1,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/北海道(1)
 {
  name => 'pst1',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 1,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/北海道(1)
 {
  name => 'pst1',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 1,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/北海道(1)
 {
  name => 'pst1',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 1,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/北海道(1)
 {
  name => 'pst1',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 1,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/青森県(2)
 {
  name => 'pst2',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 2,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/青森県(2)
 {
  name => 'pst2',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 2,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/青森県(2)
 {
  name => 'pst2',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 2,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/青森県(2)
 {
  name => 'pst2',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 2,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/青森県(2)
 {
  name => 'pst2',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 2,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岩手県(3)
 {
  name => 'pst3',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 3,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岩手県(3)
 {
  name => 'pst3',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 3,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岩手県(3)
 {
  name => 'pst3',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 3,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/岩手県(3)
 {
  name => 'pst3',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 3,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/岩手県(3)
 {
  name => 'pst3',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 3,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/宮城県(4)
 {
  name => 'pst4',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 4,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/宮城県(4)
 {
  name => 'pst4',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 4,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/宮城県(4)
 {
  name => 'pst4',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 4,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/宮城県(4)
 {
  name => 'pst4',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 4,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/宮城県(4)
 {
  name => 'pst4',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 4,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/秋田県(5)
 {
  name => 'pst5',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 5,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/秋田県(5)
 {
  name => 'pst5',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 5,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/秋田県(5)
 {
  name => 'pst5',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 5,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/秋田県(5)
 {
  name => 'pst5',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 5,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/秋田県(5)
 {
  name => 'pst5',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 5,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山形県(6)
 {
  name => 'pst6',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 6,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山形県(6)
 {
  name => 'pst6',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 6,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山形県(6)
 {
  name => 'pst6',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 6,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/山形県(6)
 {
  name => 'pst6',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 6,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/山形県(6)
 {
  name => 'pst6',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 6,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福島県(7)
 {
  name => 'pst7',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 7,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福島県(7)
 {
  name => 'pst7',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 7,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福島県(7)
 {
  name => 'pst7',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 7,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/福島県(7)
 {
  name => 'pst7',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 7,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/福島県(7)
 {
  name => 'pst7',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 7,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/茨城県(8)
 {
  name => 'pst8',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 8,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/茨城県(8)
 {
  name => 'pst8',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 8,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/茨城県(8)
 {
  name => 'pst8',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 8,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/茨城県(8)
 {
  name => 'pst8',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 8,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/茨城県(8)
 {
  name => 'pst8',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 8,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/栃木県(9)
 {
  name => 'pst9',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 9,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/栃木県(9)
 {
  name => 'pst9',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 9,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/栃木県(9)
 {
  name => 'pst9',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 9,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/栃木県(9)
 {
  name => 'pst9',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 9,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/栃木県(9)
 {
  name => 'pst9',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 9,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/群馬県(10)
 {
  name => 'pst10',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 10,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/群馬県(10)
 {
  name => 'pst10',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 10,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/群馬県(10)
 {
  name => 'pst10',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 10,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/群馬県(10)
 {
  name => 'pst10',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 10,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/群馬県(10)
 {
  name => 'pst10',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 10,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/埼玉県(11)
 {
  name => 'pst11',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 11,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/埼玉県(11)
 {
  name => 'pst11',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 11,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/埼玉県(11)
 {
  name => 'pst11',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 11,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/埼玉県(11)
 {
  name => 'pst11',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 11,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/埼玉県(11)
 {
  name => 'pst11',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 11,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/千葉県(12)
 {
  name => 'pst12',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 12,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/千葉県(12)
 {
  name => 'pst12',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 12,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/千葉県(12)
 {
  name => 'pst12',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 12,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/千葉県(12)
 {
  name => 'pst12',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 12,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/千葉県(12)
 {
  name => 'pst12',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 12,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/東京都(13)
 {
  name => 'pst13',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 13,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/東京都(13)
 {
  name => 'pst13',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 13,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/東京都(13)
 {
  name => 'pst13',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 13,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/東京都(13)
 {
  name => 'pst13',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 13,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/東京都(13)
 {
  name => 'pst13',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 13,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/神奈川県(14)
 {
  name => 'pst14',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 14,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/神奈川県(14)
 {
  name => 'pst14',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 14,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/神奈川県(14)
 {
  name => 'pst14',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 14,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/神奈川県(14)
 {
  name => 'pst14',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 14,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/神奈川県(14)
 {
  name => 'pst14',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 14,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/新潟県(15)
 {
  name => 'pst15',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 15,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/新潟県(15)
 {
  name => 'pst15',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 15,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/新潟県(15)
 {
  name => 'pst15',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 15,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/新潟県(15)
 {
  name => 'pst15',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 15,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/新潟県(15)
 {
  name => 'pst15',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 15,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/富山県(16)
 {
  name => 'pst16',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 16,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/富山県(16)
 {
  name => 'pst16',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 16,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/富山県(16)
 {
  name => 'pst16',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 16,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/富山県(16)
 {
  name => 'pst16',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 16,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/富山県(16)
 {
  name => 'pst16',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 16,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/石川県(17)
 {
  name => 'pst17',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 17,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/石川県(17)
 {
  name => 'pst17',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 17,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/石川県(17)
 {
  name => 'pst17',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 17,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/石川県(17)
 {
  name => 'pst17',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 17,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/石川県(17)
 {
  name => 'pst17',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 17,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福井県(18)
 {
  name => 'pst18',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 18,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福井県(18)
 {
  name => 'pst18',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 18,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福井県(18)
 {
  name => 'pst18',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 18,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/福井県(18)
 {
  name => 'pst18',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 18,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/福井県(18)
 {
  name => 'pst18',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 18,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山梨県(19)
 {
  name => 'pst19',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 19,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山梨県(19)
 {
  name => 'pst19',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 19,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山梨県(19)
 {
  name => 'pst19',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 19,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/山梨県(19)
 {
  name => 'pst19',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 19,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/山梨県(19)
 {
  name => 'pst19',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 19,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/長野県(20)
 {
  name => 'pst20',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 20,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/長野県(20)
 {
  name => 'pst20',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 20,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/長野県(20)
 {
  name => 'pst20',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 20,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/長野県(20)
 {
  name => 'pst20',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 20,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/長野県(20)
 {
  name => 'pst20',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 20,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岐阜県(21)
 {
  name => 'pst21',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 21,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岐阜県(21)
 {
  name => 'pst21',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 21,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岐阜県(21)
 {
  name => 'pst21',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 21,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/岐阜県(21)
 {
  name => 'pst21',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 21,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/岐阜県(21)
 {
  name => 'pst21',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 21,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/静岡県(22)
 {
  name => 'pst22',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 22,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/静岡県(22)
 {
  name => 'pst22',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 22,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/静岡県(22)
 {
  name => 'pst22',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 22,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/静岡県(22)
 {
  name => 'pst22',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 22,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/静岡県(22)
 {
  name => 'pst22',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 22,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/愛知県(23)
 {
  name => 'pst23',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 23,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/愛知県(23)
 {
  name => 'pst23',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 23,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/愛知県(23)
 {
  name => 'pst23',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 23,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/愛知県(23)
 {
  name => 'pst23',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 23,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/愛知県(23)
 {
  name => 'pst23',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 23,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 630,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/三重県(24)
 {
  name => 'pst24',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 24,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/三重県(24)
 {
  name => 'pst24',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 24,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/三重県(24)
 {
  name => 'pst24',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 24,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/三重県(24)
 {
  name => 'pst24',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 24,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/三重県(24)
 {
  name => 'pst24',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 24,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/滋賀県(25)
 {
  name => 'pst25',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 25,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/滋賀県(25)
 {
  name => 'pst25',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 25,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/滋賀県(25)
 {
  name => 'pst25',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 25,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/滋賀県(25)
 {
  name => 'pst25',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 25,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/滋賀県(25)
 {
  name => 'pst25',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 25,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/京都府(26)
 {
  name => 'pst26',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 26,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/京都府(26)
 {
  name => 'pst26',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 26,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/京都府(26)
 {
  name => 'pst26',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 26,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/京都府(26)
 {
  name => 'pst26',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 26,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/京都府(26)
 {
  name => 'pst26',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 26,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/大阪府(27)
 {
  name => 'pst27',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 27,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/大阪府(27)
 {
  name => 'pst27',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 27,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/大阪府(27)
 {
  name => 'pst27',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 27,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/大阪府(27)
 {
  name => 'pst27',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 27,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールソフト/大阪府(27)
 {
  name => 'pst27',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 27,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/兵庫県(28)
 {
  name => 'pst28',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 28,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/兵庫県(28)
 {
  name => 'pst28',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 28,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/兵庫県(28)
 {
  name => 'pst28',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 28,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/兵庫県(28)
 {
  name => 'pst28',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 28,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/兵庫県(28)
 {
  name => 'pst28',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 28,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/奈良県(29)
 {
  name => 'pst29',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 29,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/奈良県(29)
 {
  name => 'pst29',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 29,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/奈良県(29)
 {
  name => 'pst29',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 29,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/奈良県(29)
 {
  name => 'pst29',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 29,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/奈良県(29)
 {
  name => 'pst29',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 29,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/和歌山県(30)
 {
  name => 'pst30',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 30,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/和歌山県(30)
 {
  name => 'pst30',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 30,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/和歌山県(30)
 {
  name => 'pst30',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 30,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/和歌山県(30)
 {
  name => 'pst30',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 30,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/和歌山県(30)
 {
  name => 'pst30',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 30,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 840,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/鳥取県(31)
 {
  name => 'pst31',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 31,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/鳥取県(31)
 {
  name => 'pst31',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 31,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/鳥取県(31)
 {
  name => 'pst31',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 31,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/鳥取県(31)
 {
  name => 'pst31',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 31,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/鳥取県(31)
 {
  name => 'pst31',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 31,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/島根県(32)
 {
  name => 'pst32',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 32,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/島根県(32)
 {
  name => 'pst32',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 32,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/島根県(32)
 {
  name => 'pst32',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 32,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/島根県(32)
 {
  name => 'pst32',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,

			pst_are_id => 32,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/島根県(32)
 {
  name => 'pst32',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 32,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岡山県(33)
 {
  name => 'pst33',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 33,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岡山県(33)
 {
  name => 'pst33',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 33,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/岡山県(33)
 {
  name => 'pst33',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 33,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/岡山県(33)
 {
  name => 'pst33',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 33,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/岡山県(33)
 {
  name => 'pst33',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 33,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/広島県(34)
 {
  name => 'pst34',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 34,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/広島県(34)
 {
  name => 'pst34',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 34,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/広島県(34)
 {
  name => 'pst34',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 34,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/広島県(34)
 {
  name => 'pst34',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 34,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/広島県(34)
 {
  name => 'pst34',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 34,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山口県(35)
 {
  name => 'pst35',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 35,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山口県(35)
 {
  name => 'pst35',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 35,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/山口県(35)
 {
  name => 'pst35',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 35,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/山口県(35)
 {
  name => 'pst35',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 35,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/山口県(35)
 {
  name => 'pst35',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 35,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 945,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/徳島県(36)
 {
  name => 'pst36',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 36,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/徳島県(36)
 {
  name => 'pst36',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 36,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/徳島県(36)
 {
  name => 'pst36',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 36,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/徳島県(36)
 {
  name => 'pst36',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 36,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/徳島県(36)
 {
  name => 'pst36',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 36,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/香川県(37)
 {
  name => 'pst37',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 37,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/香川県(37)
 {
  name => 'pst37',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 37,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/香川県(37)
 {
  name => 'pst37',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 37,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/香川県(37)
 {
  name => 'pst37',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 37,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/香川県(37)
 {
  name => 'pst37',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 37,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/愛媛県(38)
 {
  name => 'pst38',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 38,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/愛媛県(38)
 {
  name => 'pst38',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 38,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/愛媛県(38)
 {
  name => 'pst38',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 38,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/愛媛県(38)
 {
  name => 'pst38',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 38,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/愛媛県(38)
 {
  name => 'pst38',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 38,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/高知県(39)
 {
  name => 'pst39',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 39,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/高知県(39)
 {
  name => 'pst39',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 39,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/高知県(39)
 {
  name => 'pst39',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 39,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/高知県(39)
 {
  name => 'pst39',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 39,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/高知県(39)
 {
  name => 'pst39',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 39,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福岡県(40)
 {
  name => 'pst40',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 40,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福岡県(40)
 {
  name => 'pst40',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 40,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/福岡県(40)
 {
  name => 'pst40',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 40,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/福岡県(40)
 {
  name => 'pst40',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 40,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/福岡県(40)
 {
  name => 'pst40',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 40,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/佐賀県(41)
 {
  name => 'pst41',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 41,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/佐賀県(41)
 {
  name => 'pst41',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 41,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/佐賀県(41)
 {
  name => 'pst41',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 41,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/佐賀県(41)
 {
  name => 'pst41',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 41,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/佐賀県(41)
 {
  name => 'pst41',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 41,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/長崎県(42)
 {
  name => 'pst42',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 42,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/長崎県(42)
 {
  name => 'pst42',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 42,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/長崎県(42)
 {
  name => 'pst42',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 42,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/長崎県(42)
 {
  name => 'pst42',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 42,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/長崎県(42)
 {
  name => 'pst42',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 42,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/熊本県(43)
 {
  name => 'pst43',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 43,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/熊本県(43)
 {
  name => 'pst43',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 43,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/熊本県(43)
 {
  name => 'pst43',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 43,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/熊本県(43)
 {
  name => 'pst43',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 43,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/熊本県(43)
 {
  name => 'pst43',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 43,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/大分県(44)
 {
  name => 'pst44',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 44,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/大分県(44)
 {
  name => 'pst44',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 44,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/大分県(44)
 {
  name => 'pst44',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 44,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/大分県(44)
 {
  name => 'pst44',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 44,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/大分県(44)
 {
  name => 'pst44',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 44,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/宮崎県(45)
 {
  name => 'pst45',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 45,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/宮崎県(45)
 {
  name => 'pst45',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 45,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/宮崎県(45)
 {
  name => 'pst45',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 45,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/宮崎県(45)
 {
  name => 'pst45',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 45,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/宮崎県(45)
 {
  name => 'pst45',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 45,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/鹿児島県(46)
 {
  name => 'pst46',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 46,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/鹿児島県(46)
 {
  name => 'pst46',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 46,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/鹿児島県(46)
 {
  name => 'pst46',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 46,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/鹿児島県(46)
 {
  name => 'pst46',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 46,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/鹿児島県(46)
 {
  name => 'pst46',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 46,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/沖縄県(47)
 {
  name => 'pst47',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 1,
			pst_are_id => 47,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/沖縄県(47)
 {
  name => 'pst47',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 1,
			pst_sid => 2,
			pst_are_id => 47,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # 宅配便/沖縄県(47)
 {
  name => 'pst47',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 10,
			pst_sid => 1,
			pst_are_id => 47,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 0,
			pst_icharge => 500,
			pst_ccharge => 0,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/沖縄県(47)
 {
  name => 'pst47',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 1,
			pst_are_id => 47,
			pst_price => 1,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 1050,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
 # メールギフト/沖縄県(47)
 {
  name => 'pst47',
  class => 'ESP::SMS::Data::PST',
  where => [
			pst_shp_id => 0,
			pst_dvy_id => 5,
			pst_sid => 2,
			pst_are_id => 47,
			pst_price => 10000,
			pst_number => 0,
			pst_weight => 0,
			pst_volume => 0
		   ],
  param => {
			pst_charge => 735,
			pst_icharge => 500,
			pst_ccharge => 210,
			pst_gcharge => 100,
		   },
 },
]);

$schema->output($output) if $output;
$schema->install if $install;
$schema->install_data if $install_data;
$schema->uninstall if $uninstall;

__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.

=head1 名称

 schema.pl - データベースのテーブル設定

=head1 用途

 データベースのテーブルの作成と初期データを登録します。

=head1 書式

=over 4

=item ./schema.pl --install --install_data

 テーブルが存在しない場合は作成し、存在する場合は失敗します。
 初期データが存在しない場合は登録し、存在する場合は上書きしません。

=item ./schema.pl --install --install_data --update

 テーブルが存在しない場合は作成し、存在する場合は失敗します。
 初期データが存在しない場合は登録し、存在する場合は上書きします。

=item ./schema.pl --output=docs/00schema.txt

 テーブルの構成と項目を指定したファイル(docs/00schema.txt)に書出します。

=back

=head1 補足

 schema.pl を実行するとログファイル(schema.log)が作成されます。
 エラーの詳細はログファイルをご確認下さい。

=cut
